#! /bin/sh
#
# SCSN readout module for cosmics in SM_Test
# Usage: cosmics_readout.sh [RunType]
#
#    RunType argument is optional:
#  

usage() {
    echo ""
    echo "Usage: cosmics_readout.sh NumberOfEvents"
    echo ""
}

if [ $# -gt 1 ]; then
    usage;
    exit;
fi
 
if [ $# -lt 1 ]; then
    usage;
    exit;
fi
 
e=$1

. _daq_setup
. _setup.sh

dcsreadout=/dcsnfs/lippmann/dcsreadout
temp_dir1=/tmp
temp_dir2=/dcsnfs/SM_Test/data/cosmictemp
root_prefix=/dcsnfs/SM_Test/data/cosmic-`/bin/date '+%Y%m%d-%H%M'`
log_file=temp_cosmics_readout.log

let testgood=`./modules/check_ttcvi-seq.pl | tail -1`
if [ $testgood -eq 0 ]; then
    echo ""
    echo "Found running ttcvi-seq program on daq02."
	    #echo "Maybe other test stand is running?"
    echo "Please check."
    echo ""
    exit;
fi

ev=1
while [ $ev -le $e ]; do
    echo ===============
    echo Event: $ev
    echo ===============
    # give TTC trigger
    echo "Waiting for cosmic trigger."
    ssh $trigpc "ttcvi-seq oneshot=3";

    echo -n "Reading out all chambers via scsn "

    for POS in $POS_LIST ; do
        echo -n "."
        DCS=`./modules/getdcsbypos.pl $POS`;
        DC_TYPE=`./modules/getinfobypos.pl $POS ctype`
        if [ $DC_TYPE -eq 0 ]; then
	    ssh $DCS "$dcsreadout -nothorough -l 0 -l 1 -l 2 -A  $temp_dir1/${POS}_A.bin -B $temp_dir1/${POS}_B.bin" >> $log_file 2>&1 &
	else
	    ssh $DCS "$dcsreadout -nothorough -l 0 -l 1 -l 2 -l 3 -A $temp_dir1/${POS}_A.bin -B $temp_dir1/${POS}_B.bin" >> $log_file 2>&1 &
	fi
    done
    echo ""
    wait
    for POS in $POS_LIST ; do
        DCS=`./modules/getdcsbypos.pl $POS`;
	scp $DCS:$temp_dir1/${POS}_A.bin $DCS:$temp_dir1/${POS}_B.bin $temp_dir2 &
    done
    wait
    ./ssh-sm.sh rm /tmp/*.bin
    #echo "Waiting that NFS cache is cleared."
    #sleep 30
    fname=$root_prefix-ev`printf '%04d' $ev`.root
    echo "Building event into $fname"
    ./rootlib/evbuild $temp_dir2 $fname
    rm -rf $temp_dir2/*.bin
    let ev=$ev+1 
    echo "Wrote event into $fname."
done 

echo "I wrote $1 Event(s)!"
echo ""

wait

#EOF
