#! /bin/sh
#
# Controls DCS Power
# ./dcs-power.sh layer stack
# ./dcs-power.sh report
# advanced features by C.Lippmann
#

usage() {
    echo ""
    echo " Usage: ./dcs-power.sh on                ->turn on all boards"
    echo "        ./dcs-power.sh all               ->same"
    echo "        ./dcs-power.sh off               ->turn off all boards"
    echo "        ./dcs-power.sh cycle             ->power cycle all boards"
    echo ""
    echo "        ./dcs-power.sh layer stack       ->power cycle certain dcs board(s)" 
    echo "        ./dcs-power.sh layer stack cycle ->same" 
    echo ""
    echo "        ./dcs-power.sh layer stack on    ->turn on certain board(s)"
    echo "        ./dcs-power.sh layer stack off   ->turn off certain board(s)"
    echo ""
    echo "        ./dcs-power.sh report            ->read status"
    echo "        ./dcs-power.sh read              ->same"
    echo "        ./dcs-power.sh status            ->same"
    echo ""
    echo "           [ 0 <= layer <= 5, 0 >= stack <= 4 ]"
    echo "           [ layer=all: for all layers ]"
    echo "           [ stack=all: for all stacks ]"
    echo ""
}

switchoff() {
    ssh $DCS "/usr/local/sbin/sw off 8 $LAYER $STACK; \
              sleep 1; \
              /usr/local/sbin/sw report 8; "
}

switchon() {
    ssh $DCS "/usr/local/sbin/sw on 8 $LAYER $STACK; \
              sleep 1; \
              /usr/local/sbin/sw report 8; "
}

#DCS=dcs1002
DCS=dcs1010

#echo Loading module
#ssh dcs1002 /sbin/insmod /dcsnfs/jens/psw/scomm3.o
#echo Setup power module
#ssh dcs1002 /dcsnfs/jens/psw/scw w e f

if [ $# -eq 1 ]; then
    if [ $1 == "report" ] || [ $1 == "read" ] || [ $1 == "status" ]; then
	ssh $DCS /dcsnfs/jens/psw/sw report 8
     	exit;
    elif [ $1 == "help" ] || [ $1 == "-help" ] || [ $1 == "-h" ] || [ $1 == "--help" ]; then
	usage;
	exit;
    elif [ $1 == "on" ] || [ $1 == "all" ] ; then
	LAYER=all
	STACK=all
	echo Turning on all Layers, all Stacks ...
	switchon;	
	exit;
    elif [ $1 == "off" ]; then
	LAYER=all
	STACK=all
	echo Turning off all Layers, all Stacks ...
	switchoff;
	exit;
    elif [ $1 == "cycle" ]; then
	LAYER=all
	STACK=all
	echo Turning off all Layers, all Stacks ...
	switchoff;
	echo Delay ...
	sleep 3;
	echo Turning on all Layers, all Stacks ...
	switchon;
	exit;
    else
	usage;
	exit;
    fi
elif [ $# -eq 3 ] && [ $3 == on ]; then
    LAYER=$1
    STACK=$2
    echo Turning on Layer $LAYER Stack $STACK ...
    switchon;
    exit;
elif [ $# -eq 3 ] && [ $3 == off ]; then
    LAYER=$1
    STACK=$2
    echo Turning off Layer $LAYER Stack $STACK ...
    switchoff;
    exit;
elif [ $# -eq 3 ] && [ $3 == cycle ]; then
    LAYER=$1
    STACK=$2
    echo Turning off Layer $LAYER Stack $STACK ...
    switchoff;
    echo Delay ...
    sleep 3;
    echo Turning on Layer $LAYER Stack $STACK ...
    switchon;
    exit;
elif [ $# -ne 2 ]; then
    usage;
    exit;
else 
    LAYER=$1
    STACK=$2
    echo Turning off Layer $LAYER Stack $STACK ...
    switchoff;
    sleep 3;
    echo Turning on Layer $LAYER Stack $STACK ...
    switchon;
    exit;
fi

# EOF
