#! /bin/sh
#
# SCSN readout module for SM_Test
# Usage: dcsreadout_sm.sh [RunType]
#
#    RunType argument is optional:
#    t: TTC trigger with force trigger on TTCvi.
#    c: Cosmic trigger via scintillators to TTC.
#  

usage() {
    echo ""
    echo "Usage: dcsreadout.sh [RunType] [build]"
    echo ""
    echo " RunType argument is optional:"
    echo " t: TTC trigger with force trigger on TTCvi."
    echo "or"
    echo " c: TTC trigger with force trigger on TTCvi controlled by scintillators for cosmics."
    echo ""
    echo " Build argument is optional:"
    echo " b: build event into asci file."
    echo ""
}

if [ $# -gt 2 ]; then
    usage;
    exit;
fi

if [ $# -lt 1 ]; then
    usage;
    exit;
fi

if [ $# -gt 0 ]; then
    trigger=$1
    echo "trigger type is $trigger"
    if [ x$trigger == "xt" ]; then
	let testgood=`./modules/check_ttcvi-seq.pl | tail -1`
	if [ $testgood -eq 0 ]; then
	    echo ""
	    echo "Found running ttcvi-seq program on daq02."
	    #echo "Maybe other test stand is running?"
	    echo "Please check."
	    echo ""
	    exit;
	else
	    source _daq_setup
            # give TTC trigger
	    echo "TTC pretrigger given."
	    ssh $trigpc "ttcvi-seq src=4 force=1" >> temp_dcsreadout 2>&1 &
	fi
    elif [ x$trigger == "xc" ]; then
	let testgood=`./modules/check_ttcvi-seq.pl | tail -1`
	if [ $testgood -eq 0 ]; then
	    echo ""
	    echo "Found running ttcvi-seq program on daq02."
	    #echo "Maybe other test stand is running?"
	    echo "Please check."
	    echo ""
	    exit;
	else
	    source _daq_setup
        # give TTC trigger
	    echo "TTC pretrigger given."
	    ssh $trigpc "ttcvi-seq oneshot=3" >> temp_dcsreadout 2>&1 &
        fi
    else
	usage;
	exit;
    fi
    
fi

source _setup.sh

echo -n "Reading out all chambers via scsn "

for POS in $POS_LIST ; do
    echo -n "."
    DCS=`./modules/getdcsbypos.pl $POS`;
    DC_TYPE=`./modules/getinfobypos.pl $POS ctype`
    if [ $DC_TYPE -eq 0 ]; then
	ssh $DCS "/dcsnfs/lippmann/dcsreadout -l 0 -l 1 -l 2 -A /dcsnfs/SM_Test/data/dcsreadout/${POS}_A.bin -B /dcsnfs/SM_Test/data/dcsreadout/${POS}_B.bin" \
	    >> temp_dcsreadout &
    else
	ssh $DCS "/dcsnfs/lippmann/dcsreadout -l 0 -l 1 -l 2 -l 3 -A /dcsnfs/SM_Test/data/dcsreadout/${POS}_A.bin -B /dcsnfs/SM_Test/data/dcsreadout/${POS}_B.bin" \
	    >> temp_dcsreadout &
    fi
done

echo ""

# some simple event building
if [ $# -gt 0 ] && [ x$2 == "xb" ]; then
    echo "Waiting that NFS cache is cleared."
    sleep 5
    DATAFILE="/dcsnfs/lippmann/data/dcsreadout.dat"
    echo '0xaaaaaaaa' >  $DATAFILE
    echo '0xaaaaaaaa' >> $DATAFILE
    echo -n "converting to ascii file "
    for POS in $POS_LIST ; do
	echo -n "."
	od -w4 -t x4 -v  /dcsnfs/SM_Test/data/${POS}_A.bin | cut -d " " -s -f 2 | sed "s/.*/0x&/" \
	    > /dcsnfs/lippmann/data/${POS}_A_tmp.txt
	od -w4 -t x4 -v  /dcsnfs/SM_Test/data/${POS}_B.bin | cut -d " " -s -f 2 | sed "s/.*/0x&/" \
	    > /dcsnfs/lippmann/data/${POS}_B_tmp.txt
	perl modules/remove_event_header.pl /dcsnfs/lippmann/data/${POS}_A_tmp.txt /dcsnfs/lippmann/data/${POS}_A_tmp2.txt
	perl modules/remove_end-of-event-marker.pl /dcsnfs/lippmann/data/${POS}_A_tmp2.txt /dcsnfs/lippmann/data/${POS}_A_tmp3.txt
	perl modules/remove_event_header.pl /dcsnfs/lippmann/data/${POS}_B_tmp.txt /dcsnfs/lippmann/data/${POS}_B_tmp2.txt
	perl modules/remove_end-of-event-marker.pl /dcsnfs/lippmann/data/${POS}_B_tmp2.txt /dcsnfs/lippmann/data/${POS}_B_tmp3.txt
	cat /dcsnfs/lippmann/data/${POS}_A_tmp3.txt >> $DATAFILE
	cat /dcsnfs/lippmann/data/${POS}_B_tmp3.txt >> $DATAFILE    
    done
    echo '0x00000000' >> $DATAFILE
    echo '0x00000000' >> $DATAFILE
fi

#rm -f temp_dcsreadout
rm -rf /dcsnfs/lippmann/data/*tmp*.txt

echo ""
wait

#EOF
