// Simple Event Builder
// 

#include <stdio.h>
#include <string.h>
#include <iostream>

#include "TROOT.h"
#include "TApplication.h"
#include "TCanvas.h"
#include "TFile.h"
#include "TColor.h"
#include "TStyle.h"
#include "TProfile.h"
#include "TH2.h"
#include "TH3.h"
#include "TGraph.h"
#include "TTree.h"
#include "TBranch.h"
#include "../rootlib/TRDEvent.h"

using namespace std;

TH2F* DC = new TH2F("DC","Amplitude vs Drifttime",30,0,30,72,0,72);

//_____________________________________________________________________________
Float_t GetYpos(const Float_t A0, const Float_t A1, const Float_t A2) {

//
// Get Position using gaussian PRF according to amplitudes on three pads
//

  Float_t SigP = 0.5;  // Gauss PRF sigma

  Float_t T1 = 0.0;
  Float_t W1 = 0.0;
  Float_t T2 = 0.0;
  Float_t W2 = 0.0;

  if( A0 > 0 ) {
    W1 = A0*A0;
    T1 = W1*((SigP*SigP)*TMath::Log(A1/A0)-0.5);
  }
  if( A2 > 0 ) {
    W2 = A2*A2;
    T2 = W2*((SigP*SigP)*TMath::Log(A2/A1)+0.5);
  }
  if ((W1+W2) == 0) {
    //printf("GetYpos: A0=%.2f A1=%.2f A2=%.2f ypos=%.2f \n",A0,A1,A2,(T1 + T2)/(W1 + W2));
    return -1;
  }
  return (T1 + T2)/(W1 + W2);

}

//_____________________________________________________________________________
Int_t GetPositionVectorPRF(Float_t *Tpos, Float_t *Ypos, const Int_t maxPad) {

  Float_t ClusThr = 10.0;
  Int_t   npoints = 0; 

  for (Int_t iSam = 0; iSam < 30; iSam++) {
      for (Int_t iPad = 1; iPad <= maxPad; iPad++) {

        Float_t A0 = DC->GetBinContent(iSam,iPad-1);
        Float_t A1 = DC->GetBinContent(iSam,iPad  );
        Float_t A2 = DC->GetBinContent(iSam,iPad+1);
        Float_t AA = A0 + A1 + A2;
        // cluster finding
        if (A0 > A1 || A2 > A1) continue;    // A1 is the center maximum
        if (AA < ClusThr)       continue;    // cluster threshold
        if ((A0 + A2) == 0.0)   continue;    // A0 or A1 at least must be > 0

        Float_t ypos = GetYpos(A0,A1,A2);
        if (ypos == -1.) continue;

        Tpos[npoints]  = (Float_t)iSam - 0.5;
        Ypos[npoints]  = ypos + Float_t(iPad) - 0.5;

        npoints++;
        if ( npoints >= 200 ) return -1;  

    }
  }

  return npoints;

}


int main( int argc, char *argv[] ) {

  if( argc < 2 || argc > 3 ) {
    printf("  Give rootfile as argument. Optional argument \"1\" will plot single events.\n");
    printf("  Example usage: ./modules/monitor data/cosmic-20061002-1451-ev0088.root 1\n");
    return 1;
  }

  Char_t fname[200];
  sprintf(fname, argv[1]);
  cout << "Opening file " << fname << endl;
  TFile f(fname); // Open the file

  gStyle->SetPalette(1);
  gStyle->SetOptStat(1111);
  gStyle->SetPadBorderMode(0);
  gStyle->SetCanvasColor(10);
  gStyle->SetPadLeftMargin(0.13);
  gStyle->SetPadRightMargin(0.17);
  gStyle->SetPadBottomMargin(0.13);
  gStyle->SetPadTopMargin(0.01);

  Int_t baseline = 10;
  Int_t monitor = ( argc == 3 ) ? atoi(argv[2]) : 0;

  TH2F *DCcharge = new TH2F("DCcharge","DC charge",30,0,30,100,0,200);
  DCcharge->SetStats(0);
  DCcharge->SetXTitle("Time Bin");
  DCcharge->SetYTitle("Amplitude");
  DCcharge->SetTitleOffset(1.2,"X");
  DCcharge->SetTitleOffset(1.4,"Y");

  TProfile *DCpulse = new TProfile("DCpulse","DC pulse",30,0,30);
  TGraph *tracklet = new TGraph();

  Char_t name[200];
  Char_t title[200];
  TH2F* displ[5]; // 5 stacks
  for ( Int_t istack = 0; istack < 5; istack++ ) {
    sprintf(name,  "displ%1d", istack);
    sprintf(title, "Stack %d", istack);
    displ[istack] = new TH2F(name, title, 288, -56.52, 56.52, 600, -0.5, 599.5);
    displ[istack]->SetMarkerColor(2);
    displ[istack]->SetXTitle("#phi");
    displ[istack]->SetYTitle("r (time)");
    displ[istack]->SetMarkerStyle(20);
    displ[istack]->SetMarkerSize(0.2);
    displ[istack]->SetStats(0);
  }

  //   Read Tree named "T" in memory. Tree pointer is assigned the same name
  TTree *T = (TTree*)f.Get("T");
  Int_t nev = T->GetEntries();
  //printf("Found %d events.\n", nev);
  TRDEvent *e = new TRDEvent();
  T->SetBranchAddress("TRDEvent", &e);

  printf("Reading TRDEvent...\n");
  Int_t nb = T->GetEntry( 0 );         //read complete event in memory

  TApplication theApp("Application", &argc, argv);
  TCanvas *c2;
  if (monitor) c2 = new TCanvas("c0","Canvas 0",50,50,1000,500);
  if (monitor) c2->Divide(2);

  DC->SetStats(0);
  DC->SetYTitle("Pad No.");
  DC->SetXTitle("Time Bin");
  DC->SetZTitle("Amplitude");
  DC->SetTitleOffset(1.5,"Z");

  Int_t maxsig = 0;
  Int_t iSM = 1;
  Int_t numpads = 72;

  Float_t pamp = 0;

  Float_t padw[6] = {0.635, 0.665, 0.695, 0.725, 0.755, 0.785};  // length (z) of C1 chambers

  Float_t Tpos[200];   // time position of the found clusters
  Float_t Ypos[200];   // pad position of the found clusters
  Float_t row = 0;

  printf("Filling Histograms ...\n");
  for ( int istack = 0; istack < 5; istack++) {
    int maxpadrow = ( istack == 2 ) ? 12 : 16;
    for ( int ilayer = 0; ilayer < 6; ilayer++) {
      for ( int ipadrow = 0; ipadrow < maxpadrow; ipadrow++) {
        for ( int ipad = 0; ipad < 144; ipad += numpads) {
          maxsig = 0;
          DC->Reset();
          //printf("SM%d, L%dS%d, ROB%2d, MCMs %2d-%2d\n", iSM, ilayer, istack, (ipadrow/4)*2+(ipad/numpads),
          //       (ipadrow%4)*4+((ipad/18)%4), (ipadrow%4)*4+((ipad/18)%4)+3);
          for ( int ipad2 = 0; ipad2 < numpads; ipad2++) {
            Int_t *adcdata = e->GetPadData( iSM, istack, ilayer, ipadrow, ipad+ipad2);
            // Fill Event Monitor Histogram 
            for ( int itimebin = 0 ; itimebin < 30 ; itimebin++ ) {
              DC->Fill((Float_t)itimebin, (Float_t)ipad2, (Float_t)adcdata[itimebin]-baseline);
              if ( (Int_t)(adcdata[itimebin]-baseline) > maxsig ) maxsig = (Int_t)adcdata[itimebin]-baseline;
            }
          }
          // Fill Average Signal Histogram
          for (Int_t itimebin = 0; itimebin < 30; itimebin++) {
            pamp = 0;
            for (Int_t ipad = 0; ipad < numpads; ipad++) {
              if ( DC->GetBinContent(itimebin+1, ipad+1) > 2. )
		pamp += DC->GetBinContent(itimebin+1, ipad+1);
            }
            DCcharge->Fill((Float_t)itimebin,pamp);
            DCpulse->Fill((Float_t)itimebin,pamp);
          }
	  // Tracklet reconstruction
	  Int_t nPoints = GetPositionVectorPRF(Tpos, Ypos, numpads);
	  tracklet->Set(0);
	  for (Int_t iP = 0; iP < nPoints; iP++) {
            tracklet->SetPoint(iP,Tpos[iP],Ypos[iP]);
	    displ[istack]->Fill((ipad+Ypos[iP])*padw[ilayer]-padw[ilayer]*144/2, 29-(Float_t)Tpos[iP]+60+90*ilayer);
	  }
          if ( monitor && (maxsig > 10) ) {
            sprintf(title,"SM%d, L%dS%d, ROB%2d, MCMs %2d-%2d\n", iSM, ilayer, istack,
		    (ipadrow/4)*2+( ipad/numpads), (ipadrow%4)*4+((ipad/18)%4),
		    (ipadrow%4)*4+((ipad/18)%4)+3);
            DC->SetTitle(title);
            c2->cd(1);
            DC->SetTitleOffset(1.5,"X");
            DC->SetTitleOffset(1.5,"Y");
            DC->DrawCopy("LEGO2");
            c2->cd(2);
            DC->SetTitleOffset(1.1,"X");
            DC->SetTitleOffset(1.1,"Y");
            DC->DrawCopy("COLZ");
	    tracklet->SetMarkerStyle(24);
	    tracklet->SetMarkerColor(1);
	    tracklet->SetMarkerSize(0.8);
	    if (tracklet->GetN() > 0) tracklet->Draw("P");
            c2->Update();
	    printf("Type \'q\' to quit and save image. Enter to continue.\n");
            if( getchar() == 'q' ) {
              //c2->Print("/dcsnfs/SM_Test/pics/Monitor.png","png");
              c2->Print("/dcsnfs/SM_Test/pics/Monitor.eps","eps");
              return 1;
            }
          }
        }
      }
    }
  }

  delete e;

  printf("Plotting ...\n");
  gStyle->SetPadRightMargin(0.01);
  TCanvas *c0 = new TCanvas("c0","Canvas 1",0,0,600,500);
  //TCanvas *c0 = new TCanvas("c0","Canvas 1",0,0,1200,900);
  //c0->Divide(3,2);
  //c0->cd(1);
  //DCcharge->Draw("colz");
  //DCpulse->SetLineColor(2);
  //DCpulse->SetLineWidth(2);
  //DCpulse->Draw("same");
  for ( Int_t istack = 3; istack < 4; istack++ ) {
    //c0->cd(istack+2);
    displ[istack]->Draw();
  }
  c0->cd();
  c0->Update();
  printf("Type \'q\' to quit and save image.\n");
  if( getchar() == 'q' ) {
    //c0->Print("Event.png","png");
    c0->Print("Event.eps","eps");
    return 1;
  }

}

// EOF
