#include <stdio.h>
#include <string.h>
#include <iostream>

#include "TROOT.h"
#include "TApplication.h"
#include "TCanvas.h"
#include "TFile.h"
#include "TH3.h"
#include "TTree.h"
#include "TBranch.h"
#include "../rootlib/TRDEvent.h"

using namespace std;

int main( int argc, char *argv[] ) {

  if( argc != 2) {
    printf("Give rootfile as argument.\n");
    return 1;
  }


  Char_t fname[200];
  TRDEvent *e;
  Double_t baseline = 10.;
  ROC_data *roc;

  TH3F* displ3D = new TH3F("3Ddispl", "", 92, -0.5, 91.5, 144, -0.5, 143.5, 720, -0.5, 719.5);
  displ3D->SetMarkerColor(2);
  displ3D->SetXTitle("z [padrows]");
  displ3D->SetYTitle("#phi [pads]");
  displ3D->SetZTitle("r (time) [a.u.]");

  sprintf(fname, argv[1]);
  cout << "Opening file " << fname << endl;
  TFile f(fname); // Open the file

  //   Read Tree named "T" in memory. Tree pointer is assigned the same name
  TTree *T = (TTree*)f.Get("T");
  Int_t nev = T->GetEntries();
  //printf("Found %d events.\n", nev);
  e = new TRDEvent();
  T->SetBranchAddress("TRDEvent", &e);
  //displ3D->Reset();

  Int_t nb = T->GetEntry( 0 );         //read complete event in memory

  for (int ilayer = 0; ilayer < 6; ilayer++ ) {
    for (int istack = 0; istack < 5; istack++ ) {

      roc = &(e->sm[1].roc[istack*6+ilayer]);

      // Reset histograms
      //for( Int_t ip = 0; ip < 16; ip++ ) {  evtdispl[ip]->Reset();  }

      // Fill histograms
      for( int irob = 0 ; irob < 8; irob++ ) {
	for( int imcm = 0 ; imcm < 16; imcm++ ) {
	  for( int iadc = 0 ; iadc < 21; iadc++ ) {
	    Double_t pad = (1-(irob%2)) *4*18 + (3-(imcm%4))*18 + iadc - 2;
	    if ( iadc < 2  ) pad = -1;
	      if ( iadc > 19 ) pad = -1;
	      Double_t padrow = (irob/2)*4 + imcm/4;  // pad row no
	      if (padrow < 0 || padrow > 15) printf("strange pad row number!\n");
	      Double_t glob_padrow = padrow;
	      if      ( istack<3 ) glob_padrow += istack*20;
	      else if ( istack<4 ) glob_padrow += 56;
	      else                 glob_padrow += 76;
	      for( int it = 0 ; it < 30 ; it++ ) {   // time bin loop
		if ( (roc->rob[irob].mcm[imcm].adc[iadc].tb[it] - baseline) > 10. )
		  displ3D->Fill( (Double_t) glob_padrow, (Double_t) pad, 29-(Double_t) it + 90. + 30 + 90*ilayer,
				 roc->rob[irob].mcm[imcm].adc[iadc].tb[it] - baseline);
	      }
	  }
	}
      }
    }  
  }

  printf("Plotting ...\n");
  TApplication theApp("App", &argc, argv);
  TCanvas *c2 = new TCanvas( "c2", "Cosmics", 10, 10, 1000, 400 );
  //c2->SetBorderMode(0);
  //c2->SetBorderSize(0);
  //c2->SetBottomMargin(0.);
  //c2->SetLeftMargin(0.0);
  //c2->SetRightMargin(0.);
  //c2->SetTopMargin(0.);
  c2->SetTitle("Event");
  c2->Draw();
  displ3D->DrawCopy("");
  c2->Update();
  printf("Type \'q\' to quit and save image as png file.\n");
  if( getchar()== 'q' ) {
    c2->Print("Event.png","png");
    return 1;
  }

  delete e;
  delete displ3D;
  delete c2;

  return 1;

}

// EOF
