//
//

class c_histograms {
public:
  TH2F *adc;          // RMS  map with highrst granurality

  c_histograms() {
    adc = new TH2F( "h_adc",       "ADC Raw Data",  176, -0.5, 175.5, 495, -0.5, 494.5 );
    adc->SetXTitle("pad in phi");
    adc->SetYTitle("pad/timebin");
    adc->SetZTitle("ADC");
    adc->SetAxisRange(  -10, 175, "X");
    adc->SetAxisRange(  -10, 495, "Y");
    adc->SetAxisRange( 0, 20, "Z");
  }
};

#include "rootlib/libEvent.so"

void o32_new( char *fname, int target_ev = 5, int nevents=100, char *roc_name = "LxCx-xxx",
	       char *comment = "No comment" , int chambertype = 1 )
{
  gROOT->Reset();
  gSystem->Load("rootlib/libEvent.so");

  gStyle->SetPalette(1);
  gStyle->SetOptStat(0);
  gStyle->SetPadBottomMargin(0.10);
  gStyle->SetPadLeftMargin(0.10);
  gStyle->SetPadRightMargin(0.15);
  gStyle->SetPalette(1,0);
  gStyle->SetPaintTextFormat("1.2f");

  c_histograms *h = new c_histograms();

  TFile *fout = new TFile( "o32.root", "RECREATE" );
  TTree *t   = new TTree( "T", "ROB events");
  Event *e;
  t->Branch("Event", "Event", &e, 64000, 0);

  FILE *f;
  char  stat;
  int  cmd, add, dat, slv;
  char buf[256];
  unsigned int *evdata = new unsigned int[65536*16]; // It must be big array because sometime gabbage event comes and overflow

  printf("Reading input file: %s\n", fname);
  f = fopen( fname , "r");
  if( f == NULL ) {
    printf("Failed to open file: %s\n", fname );
    exit(0);
  }

  float baseline_average = 0.;

  for( int en = 0 ; en < nevents ; en++ ) {
    e = new Event();
    int n = read_one_ev( evdata, f );
    //print_ev( evdata, n );
    printf(" en=%d\n", en );
    if( en == target_ev ) break;
    // e->PrintDigest();
    t->Fill();
    delete e;
  }


  analyze_ev( evdata, n, e );
  e->MakeDigest();
  
  // Fill histogram
  for( int irob = 0 ; irob < 8; irob++ ) {
    for( int imcm = 0 ; imcm < 16; imcm++ ) {
      for( int iadc = 0 ; iadc < 21; iadc++ ) {
	Double_t x = ((1-(irob%2)) *4 + (3-(imcm%4)))*22 + iadc;
	for( int itb = 0 ; itb < 30 ; itb++ ) {
	  Double_t y = ( (irob/2)*4 + imcm/4 ) * 31 + itb;
	  int adc = e->rob[irob].mcm[imcm].adc[iadc].tb[itb];
	  h->adc->Fill( x, y, adc );
	  // printf("irob=%d imcm=%02d iadc=%02d itb=%02d adc=%03d\n", irob, imcm, iadc, itb, adc );
	}
      }
    }
  }
  delete e;

  fclose(f);
  fout->Write();
  fout->Close();  

  TCanvas *c = new TCanvas( "c", roc_name, 100, 100, 1300, 900 );
  c->Draw();
  TPad *p_title = new TPad( "p_title", "p_title", 0, 0.95, 1.0, 1.0 );  p_title->Draw();
  TPad *p_main  = new TPad( "p_main",  "p_main", 0, 0, 1.0, 0.95 );     p_main->Draw();

  make_titlepad( p_title, fname, target_ev, roc_name, comment );

  p_main->cd();

  h->adc->Draw("COLZ");
  for( int i = 0 ; i < 8 ; i++ ) {
    TLine *l = new TLine( i*22-0.5, -0.5, 22*i-0.5, 15.5 );
    l->SetLineColor((i%4==0) ? 1 : 13); l->Draw();
  }
  for( int i = 1 ; i < 16 ; i++ ) {
    TLine *l = new TLine( -0.5, i*31-1, 167.5, i*31-1 );
    l->SetLineWidth(2);
    l->SetLineColor((i%4==3) ? 1 : 13); l->Draw();
  }

  // Write positions
  TText *text = new TText();
  for( int i = 0 ; i < 8 ; i++ ) {
    char posname[256];
    sprintf( posname, "Pos %d", i );
    text->DrawTextNDC(0.5 - 0.37*(i%2), 0.25 + (i/2)*0.20, posname);
  }

  p_main->cd(0);

  // Save graphic file
  // c->Print("bla.png","png");

  // findhighnoisepads(h);

}

// 
void make_titlepad( TPad *p_title, char *fname, int nevents, char *roc_name, char *comment )
{
  char tmp_string[1024];

  p_title->cd();

  TText *t_title = new TText( 0.02, 0.2, roc_name );
  t_title->SetTextSize( 0.8 );
  t_title->Draw();

  sprintf( tmp_string, "Comment: %s", comment );
  TText *t_comment = new TText( 0.3, 0.1, tmp_string );
  t_comment->SetTextSize( 0.4 );
  t_comment->Draw();

  sprintf( tmp_string, "File: %s", fname );
  TText *t_fname = new TText( 0.3, 0.55, tmp_string );
  t_fname->SetTextSize( 0.4 );
  t_fname->Draw();

}



//
int read_one_ev( unsigned int *evdata, FILE *f ) 
{
  int n = 0;
  char buf[256];
  unsigned int x;
  // Search next end-of-tracklet marker
  while( fgets( buf, 255, f ) != NULL ) {
    sscanf(buf, "0x%x", &x);
    if( x == 0xaaaaaaaa ) {
      printf("End-of-Tracklet\n");
      break;
    }
  }
  // Search non-EOT marker
  while( fgets( buf, 255, f ) != NULL ) {
    sscanf(buf, "0x%x", &x);
    if( x != 0xaaaaaaaa ) break;
  }
  evdata[n++] = x;
  // Copy till end of event
  while( fgets( buf, 255, f ) != NULL ) {
    sscanf(buf, "0x%x", &x);
    if( x == 0x00000000 ) break;
    evdata[n++] = x;
  }
  printf("n=%d\n", n);
  return n;
}

void analyze_ev( unsigned int *evdata, int n, Event *e )
{
  int nROB, nMCM;
  int j = 0;
  unsigned int x;
  unsigned int a0, a1, a2;
  if( (n % 3376) != 0 ) { printf("Data is strange (size = %d).\n", n); }
  unsigned int iROB, iMCM, iRest, iEv, iADC=0, iT=0;
  for( j = 0 ; j < n ; j++ ) {
    x = evdata[j];
    if( (x & 0x0000000f) == 0xC){
      iMCM  = (x & 0xff000000) >> 24;
      iEv   = (x & 0x00fffff0) >> 4;
      iRest = (x & 0x0000000f);
      iROB  = iMCM / 16;
      iMCM  = iMCM % 16;      e->EventNum = iEv;
      //printf("iROB=%d iMCM=%02d iEv=%02d\n", iROB, iMCM, iEv );
      iADC = 0; // Reset ADC counter
      iT   = 0;
    } else {
      if( j == 0 )    { printf("Data is strange. j=0 but ADC data.\n"); continue; }
      if( iADC > 20 ) { printf("Data is strange. iADC is already %d\n", iADC); continue; }
      a0 = (x & 0x00000ffc) >> 2;
      a1 = (x & 0x003ff000) >> 12;
      a2 = (x & 0xffc00000) >> 22;
      e->FillAdcData( iROB, iMCM, iADC, iT++, a0 );
      e->FillAdcData( iROB, iMCM, iADC, iT++, a1 );
      e->FillAdcData( iROB, iMCM, iADC, iT++, a2 );
      // printf("ADC: %d  %04d %04d %04d\n", iADC, a0, a1, a2);
      if( iT == 30 ) {
	iADC++;
	iT=0;
      }
    }
  }
}

void findhighnoisepads(c_histograms *h) 
{
  
  

  return;
}


// EOF
