//
//

class c_histograms {
public:
  TH2F *rms;          // RMS  map with highrst granurality
  TH2F *mean;         // Mean map with highrst granurality
  TH2F *rms_mcm;      // RMS  map with MCM-by-MCM granurality
  TH2F *rms_rob;      // RMS  map with ROB-by-ROB granurality
  TH1F *rms_pads;     // RMS map with trace specific features

  c_histograms( Float_t range_rms_l, Float_t range_rms_h, Float_t range_bsl_l, Float_t range_bsl_h ) {

    rms       = new TH2F( "h_rms",       "RMS distribution in channels",  168, -0.5, 167.5, 16, -0.5, 15.5 );
    mean      = new TH2F( "h_mean",      "Mean distribution of baseline", 168, -0.5, 167.5, 16, -0.5, 15.5 );
    rms_mcm   = new TH2F( "h_rms_mcm",   "RMS distribution (MCM)",          8, -0.5,   7.5, 16, -0.5, 15.5 );
    rms_rob   = new TH2F( "h_rms_rob",   "RMS distribution (ROB)",          2, -0.5,   1.5,  4, -0.5,  3.5 );
    rms_pads  = new TH1F( "h_rms_pads",  "RMS distribution (Pads)",       144, -0.5, 143.5 );

    rms->SetXTitle("ADC channel number in phi");
    rms->SetYTitle("Chamber Pad number in z");
    rms->SetZTitle("RMS");
    rms->SetAxisRange(  -10, 168, "X");
    rms->SetAxisRange(  -10,  16, "Y");
    rms->SetAxisRange( range_rms_l, range_rms_h, "Z");

    mean->SetXTitle("ADC channel Number in phi");
    mean->SetYTitle("Chamber Pad Number in z");
    mean->SetZTitle("Mean");
    mean->SetAxisRange( -10, 168, "X");
    mean->SetAxisRange( -10,  16, "Y");
    mean->SetAxisRange( range_bsl_l, range_bsl_h, "Z");

    rms_mcm->SetXTitle("MCM Number in phi");
    rms_mcm->SetYTitle("MCM Number in z");
    rms_mcm->SetZTitle("<RMS>");
    rms_mcm->SetAxisRange( range_rms_l, range_rms_h, "Z");
    rms_mcm->SetMarkerSize(2);

    rms_rob->SetXTitle("ROB Number in phi");
    rms_rob->SetYTitle("ROB Number in z");
    rms_rob->SetZTitle("<RMS>");
    rms_rob->SetAxisRange( range_rms_l, range_rms_h, "Z");
    rms_rob->SetMarkerSize(3);

    rms_pads->SetXTitle("ADC channel number in phi");
    rms_pads->SetYTitle("RMS");
    rms_pads->SetAxisRange( -10, 168, "X");
    rms_pads->SetAxisRange(0.05, 12., "Y");
    rms_pads->SetFillColor(2);
  }
};

#include "rootlib/libTRDEvent.so"

void o32_plot( char *fname, int nevents = 1, char *roc_name = "LxCx-xxx",
	       char *comment = "No comment" , int chambertype = 1,
	       Float_t range_rms_l = 0.5, Float_t range_rms_h = 2.5,    // RMS plot's range
	       Float_t range_bsl_l = 5.0, Float_t range_bsl_h = 15.0 )  // Baseline plot's range
{
  gROOT->Reset();
  gSystem->Load("rootlib/libTRDEvent.so");

  gStyle->SetPalette(1);
  gStyle->SetOptStat(0);
  gStyle->SetPadBottomMargin(0.10);
  gStyle->SetPadLeftMargin(0.10);
  gStyle->SetPadRightMargin(0.15);
  gStyle->SetPalette(1,0);
  gStyle->SetPaintTextFormat("1.2f");

  c_histograms *h = new c_histograms( range_rms_l, range_rms_h, range_bsl_l, range_bsl_h );

  TFile *fout = new TFile( "o32.root", "RECREATE" );
  TTree *t   = new TTree( "T", "ROB events");
  TRDEvent *e;
  t->Branch("TRDEvent", "TRDEvent", &e, 64000, 0);

  FILE *f;
  char  stat;
  int  nline = 0;
  int  cmd, add, dat, slv;
  char buf[256];
  unsigned int *evdata = new unsigned int[65536*16]; // It must be big array because sometime gabbage event comes and overflow

  printf("Reading input file: %s\n", fname);
  f = fopen( fname , "r");
  if( f == NULL ) {
    printf("Failed to open file: %s\n", fname );
    exit(0);
  }

  float baseline_average = 0.;

  for( int en = 0 ; en < nevents ; en++ ) {
    e = new TRDEvent();
    int n = read_one_ev( evdata, f );
    //print_ev( evdata, n );
    printf(" en=%d\n", en );
    analyze_ev( evdata, n, e );
    e->MakeDigest();
    // Fill histogram
    for( int irob = 0 ; irob < 8; irob++ ) {
      Double_t rms_rob = 0;
      Double_t mean_rob = 0;
      Double_t nmcm = 0;
      for( int imcm = 0 ; imcm < 16; imcm++ ) {
	Double_t rms_mcm = 0;
	Double_t mean_mcm = 0;
	Double_t nadc = 0;
	for( int iadc = 0 ; iadc < 21; iadc++ ) {
	  Double_t rms  = e->sm[1].roc[1].rob[irob].mcm[imcm].adc[iadc].rms;
	  Double_t mean = e->sm[1].roc[1].rob[irob].mcm[imcm].adc[iadc].mean;
	  Double_t x = (1-(irob%2)) *4*21 + (3-(imcm%4))*21 + iadc;
	  Double_t pad = (1-(irob%2)) *4*18 + (3-(imcm%4))*18 + iadc - 2;
	  if ( iadc < 2 )  pad = -1;
	  if ( iadc > 19 ) pad = -1;
	  Double_t y = (irob/2)*4 + imcm/4;
	  //printf("iROB=%d iMCM=%d, iADC=%d, X=%f, Y=%f, RMS=%f, Mean=%f\n", irob, imcm, iadc, x, y, rms, mean );
	  h->rms     ->Fill( x, y, rms /(Double_t)nevents );
	  h->rms_pads->Fill( pad,  rms /16./(Double_t)nevents );
	  h->mean    ->Fill( x, y, mean/(Double_t)nevents );
	  if( iadc > 1 && iadc < 20 && rms > 0.01 ) {
	    rms_mcm  += rms;
	    mean_mcm += mean;
	    nadc    += 1.0;
	  }
	}
	if( nadc > 0 ) { rms_mcm  /= nadc; mean_mcm /= nadc; } else { rms_mcm = mean_mcm = 0; }
	//printf("ROB %d MCM %02d <RMS>=%f\n", irob, imcm, rms_mcm );
	h->rms_mcm->Fill( (1-(irob%2)) *4 + (3-(imcm%4)), (irob/2)*4 + imcm/4, rms_mcm/(Double_t)nevents );
	if( rms_mcm > 0.01 ) {
	  rms_rob  += rms_mcm;
	  mean_rob += mean_mcm;
	  nmcm     += 1.0; 
	}
      }
      if( nmcm > 0 ) { rms_rob  /= nmcm; mean_rob /= nmcm; } else { rms_rob = mean_rob = 0; }
      baseline_average += mean_rob;
      printf("ROB %d: <Baseline>= %f, <RMS>=%f\n", irob, mean_rob, rms_rob );
      h->rms_rob->Fill( (1-(irob%2)), (irob/2), rms_rob/(Double_t)nevents );
    }

    // e->PrintDigest();
    t->Fill();
    delete e;
  }

  fclose(f);
  printf("Number of lines read: %d\n", nline);
  fout->Write();
  fout->Close();  

  TCanvas *c = new TCanvas( "c", roc_name, 100, 100, 1300, 900 );
  c->Draw();
  TPad *p_title = new TPad( "p_title", "p_title", 0, 0.95, 1.0, 1.0 );  p_title->Draw();
  TPad *p_main  = new TPad( "p_main",  "p_main", 0, 0, 1.0, 0.95 );     p_main->Draw();

  make_titlepad( p_title, fname, nevents, roc_name, comment );

  p_main->cd();
  p_main->Divide(2, 2);

  p_main->cd(1);
  h->mean->Draw("COLZ");
  for( int i = 0 ; i < 8 ; i++ ) {
    TLine *l = new TLine( i*21-0.5, -0.5, 21*i-0.5, 15.5 );
    l->SetLineColor((i%4==0) ? 1 : 13); l->Draw();
  }
  for( int i = 0 ; i < 16 ; i++ ) {
    TLine *l = new TLine( -0.5, i+0.5, 167.5, i+0.5 );
    l->SetLineColor((i%4==3) ? 1 : 13); l->Draw();
  }

  p_main->cd(2);
  h->rms->Draw("COLZ");
  for( int i = 0 ; i < 8 ; i++ ) {
    TLine *l = new TLine( i*21-0.5, -0.5, 21*i-0.5, 15.5 );
    l->SetLineColor((i%4==0) ? 1 : 13); l->Draw();
  }
  for( int i = 0 ; i < 16 ; i++ ) {
    TLine *l = new TLine( -0.5, i+0.5, 167.5, i+0.5 );
    l->SetLineColor((i%4==3) ? 1 : 13); l->Draw();
  }

  p_main->cd(3);
  h->rms_mcm->Draw("COLZ");
  h->rms_mcm->Draw("TEXTSAME");
  // Write grid myself
  for( int i = 0 ; i < 7 ; i++ ) {
    TLine *l = new TLine( i+0.5, -0.5, i+0.5, 15.5 );
    l->SetLineColor((i%4==3) ? 1 : 13); l->Draw();
  }
  for( int i = 0 ; i < 16 ; i++ ) {
    TLine *l = new TLine( -0.5, i+0.5, 7.5, i+0.5 );
    l->SetLineColor((i%4==3) ? 1 : 13); l->Draw();
  }

  p_main->cd(4);
  float chamber_average = 0.;
  for (int z = 1; z < 5; z++) {
    chamber_average += h->rms_rob->GetBinContent(1,z);
    chamber_average += h->rms_rob->GetBinContent(2,z);
  }
  char buffer [100];
  sprintf(buffer, "RMS distribution (ROB),      chamber average = %1.2f", chamber_average/(2.*(chambertype+3)));
  //printf("Chamber average Noise:    %1.2f\n", chamber_average/(2.*(chambertype+3)));
  //printf("Chamber average Baseline: %1.2f\n", baseline_average/(Double_t)nevents/(2.*(chambertype+3)));
  printf("Chamber average Noise:    %1.2f\6", chamber_average/(2.*(chambertype+3)));
  printf("Chamber average Baseline: %1.2f\6", baseline_average/(Double_t)nevents/(2.*(chambertype+3)));
  h->rms_rob->SetTitle(buffer);
  h->rms_rob->Draw("COLZ");
  h->rms_rob->Draw("TEXTSAME");
  // Write grid myself
  for( int i = 0 ; i < 2 ; i++ ) {
    TLine *l = new TLine( i+0.5, -0.5, i+0.5, 3.5 );
    l->SetLineColor(13); l->Draw();
  }
  for( int i = 0 ; i < 4 ; i++ ) {
    TLine *l = new TLine( -0.5, i+0.5, 1.5, i+0.5 );
    l->SetLineColor(13); l->Draw();
  }

  // Write positions
  TText *text = new TText();
  for( int i = 0 ; i < 8 ; i++ ) {
    char posname[256];
    sprintf( posname, "Pos %d", i );
    text->DrawTextNDC(0.5 - 0.37*(i%2), 0.25 + (i/2)*0.20, posname);
  }

  p_main->cd(0);

  find_bad_channels(h, range_rms_h, range_bsl_h );

  // Save graphic file
  TString outfile;
  outfile = "pics/";
  outfile += fname;
  //if( strcmp( outfile + strlen(outfile)-4, ".txt" ) == 0 ) {
  //  strcpy( outfile +  strlen(outfile)-4, ".png" );
  //} else {
  //strcpy( outfile, "bla.png" );
  //}
  c->Print( outfile );
}

// 
void make_titlepad( TPad *p_title, char *fname, int nevents, char *roc_name, char *comment )
{
  char tmp_string[1024];

  p_title->cd();

  TText *t_title = new TText( 0.02, 0.2, roc_name );
  t_title->SetTextSize( 0.8 );
  t_title->Draw();

  sprintf( tmp_string, "Comment: %s", comment );
  TText *t_comment = new TText( 0.3, 0.1, tmp_string );
  t_comment->SetTextSize( 0.4 );
  t_comment->Draw();

  sprintf( tmp_string, "File: %s", fname );
  TText *t_fname = new TText( 0.3, 0.55, tmp_string );
  t_fname->SetTextSize( 0.4 );
  t_fname->Draw();

}



//
int read_one_ev( unsigned int *evdata, FILE *f ) 
{
  int n = 0;
  char buf[256];
  unsigned int x;
  // Search next end-of-tracklet marker
  while( fgets( buf, 255, f ) != NULL ) {
    sscanf(buf, "0x%x", &x);
    if( x == 0xaaaaaaaa ) {
      printf("End-of-Tracklet\n");
      break;
    }
  }
  // Search non-EOT marker
  while( fgets( buf, 255, f ) != NULL ) {
    sscanf(buf, "0x%x", &x);
    if( x != 0xaaaaaaaa ) break;
  }
  evdata[n++] = x;
  // Copy till end of event
  while( fgets( buf, 255, f ) != NULL ) {
    sscanf(buf, "0x%x", &x);
    if( x == 0x00000000 ) break;
    evdata[n++] = x;
  }
  printf("n=%d\n", n);
  return n;
}

void analyze_ev( unsigned int *evdata, int n, TRDEvent *e )
{
  int nROB, nMCM;
  int j = 0;
  unsigned int x;
  unsigned int a0, a1, a2;
  if( (n % 3376) != 0 ) { printf("Data is strange, size is smaller than normal (%d).\n", n); }
  unsigned int iROB, iMCM, iRest, iEv, iADC=0, iT=0;
  for( j = 0 ; j < n ; j++ ) {
    x = evdata[j];
    if( (x & 0x0000000f) == 0xC){ // MCM Header
      iMCM  = (x & 0xff000000) >> 24;
      iEv   = (x & 0x00fffff0) >> 4;
      iRest = (x & 0x0000000f);
      iROB  = iMCM / 16;
      iMCM  = iMCM % 16;      e->EventNum = iEv;
      //printf("iROB=%d iMCM=%02d iEv=%02d\n", iROB, iMCM, iEv );
      iADC = 0; // Reset ADC counter
      iT   = 0;
    } else {
      if( j == 0 )    { printf("Data is strange. j=0 but ADC data.\n"); continue; }
      if( iADC > 20 ) { printf("Data is strange. iADC is already %d\n", iADC); continue; }
      a0 = (x & 0x00000ffc) >> 2;
      a1 = (x & 0x003ff000) >> 12;
      a2 = (x & 0xffc00000) >> 22;
      e->FillAdcData( 1, 1, iROB, iMCM, iADC, iT++, a0 );
      e->FillAdcData( 1, 1, iROB, iMCM, iADC, iT++, a1 );
      e->FillAdcData( 1, 1, iROB, iMCM, iADC, iT++, a2 );
      //printf("ADC: %d  %04d %04d %04d\n", iADC, a0, a1, a2);
      if( iT == 30 ) {
	iADC++;
	iT=0;
      }
    }
  }
}

// Find automatically high noise channels and high pedestal channels.
//
void find_bad_channels(c_histograms *h, Double_t rms_thresh, Double_t mean_thresh ) 
{
  printf("Scanning channels.\n");
  printf("Threshold: %.2f for RMS\n",     rms_thresh );
  printf("Threshold: %.2f for Pedestal\n", mean_thresh );

  printf("// begin tcs\n" );

  for( int irob = 0 ; irob < 8; irob++ ) {
    for( int imcm = 0 ; imcm < 16; imcm++ ) {
      int ADCMSK = 0x1FFFFF;
      if( ((irob % 2) == 0 ) && ((imcm % 4) == 0)) ADCMSK = 0x0FFFFF;
      if( ((irob % 2) == 1 ) && ((imcm % 4) == 3)) ADCMSK = 0x1FFFFC;
      int modify = 0;
      for( int iadc = 0 ; iadc < 21; iadc++ ) {
	Int_t x = (1-(irob%2)) *4*21 + (3-(imcm%4))*21 + iadc +1;
	Int_t y = (irob/2)*4 + imcm/4 +1;
	Double_t rms  = h->rms ->GetBinContent( x, y );
	Double_t mean = h->mean->GetBinContent( x, y );
	if( rms > rms_thresh ) {
	  modify ++;
	  ADCMSK = ADCMSK & (~(1 << iadc));
	  printf("// highnoise: irob=%d, imcm=%2d iadc=%d rms=%.2f\n", irob, imcm, iadc, rms);
	}
	if( mean > mean_thresh ) {
	  modify ++;
	  ADCMSK = ADCMSK & (~(1 << iadc));
	  printf("// highnoise: irob=%d, imcm=%2d iadc=%d mean=%.2f\n", irob, imcm, iadc, mean);
	}

      }
      if( modify != 0 ) {
	printf("write chip_rob%d | chip%d, ADCMSK, 0x%x;\n", irob, imcm, ADCMSK);
      }
    }
  }
  return;
}


// EOF
