#! /bin/sh
#
# Optical readout module for SM_Test (only for standalone chamber)
# Usage: opt_readout.sh Position RunType NEvents [Comment]
#
#    s: DCS software trigger, for noise measurement, TTC is not used.
#    t: TTC trigger with force trigger on TTCvi.
#
#    Comment is optional.

if [ $# -lt 2 ] || [ $# -gt 3 ]; then
    echo ""
    echo "Usage: opt_readout.sh Position RunType NEvents [Comment]"
    echo ""
    echo "    s: DCS software trigger, for noise measurement, TTC is not used."
    echo "    t: TTC trigger with force trigger on TTCvi."
    echo "    Comment is optional."
    echo ""
    exit;
fi

# Check arguments
POS=$1
trigger=$2
nev=$3

dcs=`./modules/getdcsbypos.pl $POS`
DC_NAME=`./modules/getinfobypos.pl $POS rocname`
DC_TYPE=`./modules/getinfobypos.pl $POS ctype`

SM=`./modules/getinfobypos.pl $POS sm`
LAYER=`./modules/getinfobypos.pl $POS layer`
STACK=`./modules/getinfobypos.pl $POS stack`

. logs/last_config_$POS
echo Configuration: $LAST_CONFIG

#dcs=dcs0490

if [ x$trigger == "xt" ]; then
    let testgood=`./modules/check_ttcvi-seq.pl | tail -1`
    if [ $testgood -eq 0 ]; then
	echo ""
	echo "Found running ttcvi-seq program on daq02."
	echo "Maybe other test stand is running?"
	echo "Please check."
	echo ""
	exit;
    fi
fi
if [ x$trigger != "xs" ] && [ x$trigger != "xt" ] && [ x$trigger != "xc" ] ; then 
    echo "Unknown trigger type $trigger ."
    exit;
fi
if [ -z $nev ]; then 
    echo "Error: give number of events to take as second argument."
    exit;
fi

source _daq_setup

# Define output file
date=`/bin/date '+%Y%m%d-%H%M'`
file=data/data-$date-SM${SM}L${LAYER}S${STACK}-$LAST_CONFIG.txt

echo DCS: $dcs
echo Data file: $file

if [ x$trigger == "xs" ] ; then
    echo Setting trigger type to s
    ssh $dcs "/usr/local/sbin/ttcrx_regs s"
fi

# Flush garbage data
ssh $daqpc "bin/ninew -$card0 -r; bin/ninew -$card1 -r;"

# Take events
ev=1
while [ $ev -le $nev ]; do
  echo ------------------------------ Event: $ev ------------------------------
  if [ $trigger == "s" ]; then 
      ssh $dcs "/usr/local/sbin/pretrigger 1"
  elif [ $trigger == "i" ]; then
      ssh $dcs "/usr/local/sbin/pretrigger 7"
  elif [ $trigger == "c" ]; then
      ssh $trigpc "ttcvi-seq oneshot=3"
  else
      ssh $trigpc "ttcvi-seq src=4 force=1"
  fi

  ssh $daqpc "bin/ninew -$card0 -o32 temp0.out -r; \
              bin/ninew -$card1 -o32 temp1.out -r;"
  scp $daqpc:'temp0.out' .
  scp $daqpc:'temp1.out' .

  perl modules/remove_event_header.pl temp0.out temp0a.out
  perl modules/remove_end-of-event-marker.pl temp0a.out temp0b.out
  perl modules/remove_event_header.pl temp1.out temp1a.out
  perl modules/remove_end-of-event-marker.pl temp1a.out temp1b.out

  echo '0xaaaaaaaa' >> $file
  echo '0xaaaaaaaa' >> $file
  cat temp0b.out >> $file
  cat temp1b.out >> $file
  echo '0x00000000' >> $file
  echo '0x00000000' >> $file

  ev=$(($ev+1))
done

rm -f temp0.out temp1.out temp0a.out temp1a.out temp0b.out temp1b.out
ln -sf $file _last_data
echo ------------------------------ D  o  n  e ------------------------------
echo ""
echo "Done. Data was stored in: $file"
echo ""

# EOF
