//
//
// Now it reads directory directly

#include "rootlib/libTRDEvent.so"

void plot_sm( TString fdname, int nevents = 1, char *roc_name = "LxCx-xxx", int ilayer = 0,
	      int istack = 0, char *comment = "No comment", int drawtype = 0,
	      Float_t range_rms_l = 0.5, Float_t range_rms_h = 2.5,    // RMS plot's range
	      Float_t range_bsl_l = 5.0, Float_t range_bsl_h = 15.0 )  // Baseline plot's range
{
  gROOT->Reset();
  gSystem->Load("rootlib/libTRDEvent.so");

  TRDEvent *e = new TRDEvent();

  if ( fdname.EndsWith(".root")) {
    // Read data from rootfile
    TFile f(fdname); // Open the file
    //   Read Tree named "T" in memory. Tree pointer is assigned the same name
    T = (TTree*)f.Get("T");
    Int_t nev = T->GetEntries();
    printf("Found %d event(s).\n", nev);
    T->SetBranchAddress("TRDEvent", &e);
    Int_t nb = T->GetEntry( 0 );         // read complete event in memory
    f.Close();
  } else {
    // Read data from directory structure
    e->ReadDCSFiles( 1, fdname.Data() );  // Read all files in the directory into class
    e->MakeDigest();               // Calculate mean and so on
    // e->PrintDigest();           // Dump all mean and rms, status
  }
  if ( (ilayer == 99) && (istack == 99) )
    Analyse_SM( e, nevents, comment, drawtype, range_rms_l, range_rms_h,
		range_bsl_l, range_bsl_h, fdname);
  else
    Analyse_ROC( e, nevents, roc_name, ilayer, istack , comment, 
		 range_rms_l, range_rms_h, range_bsl_l, range_bsl_h, fdname );
  
}

////////////////////////////////////////////////////
// Analyse a single ROC in SM
////////////////////////////////////////////////////

void Analyse_ROC( TRDEvent *e, int nevents = 1, char *roc_name = "LxCx-xxx", int ilayer = 0,
		  int istack = 0, char *comment = "No comment", 
		  Float_t range_rms_l = 0.5, Float_t range_rms_h = 2.5,    // RMS plot's range
		  Float_t range_bsl_l = 5.0, Float_t range_bsl_h = 15.0,  // Baseline plot's range
		  char *prefix="bla" ) // Prefix for output filenames
{

  gStyle->SetOptStat(0);
  gStyle->SetPadTopMargin(0.05);
  gStyle->SetPadLeftMargin(0.08);
  gStyle->SetPadRightMargin(0.13);
  gStyle->SetPadBottomMargin(0.08);
  gStyle->SetPadBorderSize(0.);
  gStyle->SetFrameBorderMode(0);
  gStyle->SetFrameBorderSize(0.);
  gStyle->SetPalette(1,0);
  gStyle->SetPaintTextFormat("1.2f");

  int chambertype = (istack == 2) ? 0 : 1;
  int iroc = istack*6+ilayer;
  c_histograms *h = new c_histograms( range_rms_l, range_rms_h, range_bsl_l, range_bsl_h );
  char buf[256];
  ROC_data *roc = &(e->sm[1].roc[istack*6+ilayer]);

  float roc_bl_average = 0.;
  float roc_noise_average = 0.;
  
  // Fill histogram
  for( int irob = 0 ; irob < 8; irob++ ) {
    Double_t rms_rob = 0;
    Double_t mean_rob = 0;
    Double_t nmcm = 0;
    for( int imcm = 0 ; imcm < 16; imcm++ ) {
      Double_t rms_mcm = 0;
      Double_t mean_mcm = 0;
      Double_t nadc = 0;
      for( int iadc = 0 ; iadc < 21; iadc++ ) {
	Double_t rms  = roc->rob[irob].mcm[imcm].adc[iadc].rms;
	Double_t mean = roc->rob[irob].mcm[imcm].adc[iadc].mean;
	Double_t p2p  = roc->rob[irob].mcm[imcm].adc[iadc].peaktopeak;
	Double_t x = (1-(irob%2)) *4*21 + (3-(imcm%4))*21 + iadc;
	Double_t pad = (1-(irob%2)) *4*18 + (3-(imcm%4))*18 + iadc - 2;
	if ( iadc < 2 )  pad = -1;
	if ( iadc > 19 ) pad = -1;
	Double_t y = (irob/2)*4 + imcm/4;
	//printf("iROB=%d iMCM=%d, iADC=%d, X=%f, Y=%f, RMS=%f, Mean=%f\n", irob, imcm, iadc, x, y, rms, mean );
	h->rms     ->Fill( x, y, rms /(Double_t)nevents );
	h->rms_pads->Fill( pad,  rms /16./(Double_t)nevents );
	h->mean    ->Fill( x, y, mean/(Double_t)nevents );
	h->p2p     ->Fill( x, y, p2p/(Double_t)nevents );
	if( iadc > 1 && iadc < 20 && rms > 0.01 ) {
	  rms_mcm  += rms;
	  mean_mcm += mean;
	  nadc    += 1.0;
	}
      }
      if( nadc > 0 ) { rms_mcm  /= nadc; mean_mcm /= nadc; } else { rms_mcm = mean_mcm = 0; }
      //printf("ROB %d MCM %02d <RMS>=%f\n", irob, imcm, rms_mcm );
      h->rms_mcm->Fill( (1-(irob%2)) *4 + (3-(imcm%4)), (irob/2)*4 + imcm/4, rms_mcm/(Double_t)nevents );
      if( rms_mcm > 0.01 ) {
	rms_rob  += rms_mcm;
	mean_rob += mean_mcm;
	nmcm     += 1.0; 
      }
    }
    if( nmcm > 0 ) { rms_rob  /= nmcm; mean_rob /= nmcm; } else { rms_rob = mean_rob = 0; }
    printf("ROB %d: <Baseline>= %.2f, <RMS>=%.2f\n", irob, mean_rob, rms_rob );
    h->rms_rob->Fill( (1-(irob%2)), (irob/2), rms_rob/(Double_t)nevents );
    
    roc_bl_average    += mean_rob;
    roc_noise_average += rms_rob;

  }

  TCanvas *c = new TCanvas( "c", roc_name, 0, 0, 1300, 900 );
  c->Draw();
  TPad *p_title = new TPad( "p_title", "p_title", 0, 0.95, 1.0, 1.0 );  p_title->Draw();
  TPad *p_main  = new TPad( "p_main",  "p_main", 0, 0, 1.0, 0.95 );     p_main->Draw();

  make_titlepad( p_title, prefix, nevents, roc_name, comment );

  p_main->cd();
  p_main->Divide(2, 2);

  p_main->cd(1);
  h->mean->Draw("COLZ");
  for( int i = 0 ; i < 8 ; i++ ) {
    TLine *l = new TLine( i*21-0.5, -0.5, 21*i-0.5, 15.5 );
    l->SetLineColor((i%4==0) ? 1 : 13); l->Draw();
  }
  for( int i = 0 ; i < 16 ; i++ ) {
    TLine *l = new TLine( -0.5, i+0.5, 167.5, i+0.5 );
    l->SetLineColor((i%4==3) ? 1 : 13); l->Draw();
  }

  p_main->cd(2);
  h->rms->Draw("COLZ");
  for( int i = 0 ; i < 8 ; i++ ) {
    TLine *l = new TLine( i*21-0.5, -0.5, 21*i-0.5, 15.5 );
    l->SetLineColor((i%4==0) ? 1 : 13); l->Draw();
  }
  for( int i = 0 ; i < 16 ; i++ ) {
    TLine *l = new TLine( -0.5, i+0.5, 167.5, i+0.5 );
    l->SetLineColor((i%4==3) ? 1 : 13); l->Draw();
  }

  p_main->cd(3);
  h->rms_mcm->Draw("COLZ");
  h->rms_mcm->Draw("TEXTSAME");
  // Write grid myself
  for( int i = 0 ; i < 7 ; i++ ) {
    TLine *l = new TLine( i+0.5, -0.5, i+0.5, 15.5 );
    l->SetLineColor((i%4==3) ? 1 : 13); l->Draw();
  }
  for( int i = 0 ; i < 16 ; i++ ) {
    TLine *l = new TLine( -0.5, i+0.5, 7.5, i+0.5 );
    l->SetLineColor((i%4==3) ? 1 : 13); l->Draw();
  }

  p_main->cd(4);
  float chamber_average = 0.;
  for (int z = 1; z < (chambertype+4); z++) {
    chamber_average += h->rms_rob->GetBinContent(1,z);
    chamber_average += h->rms_rob->GetBinContent(2,z);
  }
  char buffer [100];
  sprintf(buffer, "RMS distribution (ROB),      chamber average = %1.2f", chamber_average/(2.*(chambertype+3)));

  printf("==========================================\n");
  printf("ROC  : <Baseline>= %.2f, <RMS>=%.2f\n", roc_bl_average/(2.*(chambertype+3)),
	 roc_noise_average/(2.*(chambertype+3)) );

  h->rms_rob->SetTitle(buffer);
  h->rms_rob->Draw("COLZ");
  h->rms_rob->Draw("TEXTSAME");
  // Write grid myself
  for( int i = 0 ; i < 2 ; i++ ) {
    TLine *l = new TLine( i+0.5, -0.5, i+0.5, 3.5 );
    l->SetLineColor(13); l->Draw();
  }
  for( int i = 0 ; i < 4 ; i++ ) {
    TLine *l = new TLine( -0.5, i+0.5, 1.5, i+0.5 );
    l->SetLineColor(13); l->Draw();
  }

  // Write positions
  TText *text = new TText();
  for( int i = 0 ; i < 8 ; i++ ) {
    char posname[256];
    sprintf( posname, "Pos %d", i );
    text->DrawTextNDC(0.5 - 0.37*(i%2), 0.25 + (i/2)*0.20, posname);
  }

  p_main->cd(0);

  find_bad_channels(h, range_rms_h, range_bsl_h );

  // Save graphic file
  char outfile[1024];
  sprintf( outfile, "%s/roc.png", prefix );
  c->Print( outfile );
}


////////////////////////////////////////////////////
// Analyse all ROCs in SM
////////////////////////////////////////////////////

void Analyse_SM( TRDEvent *e, int nevents = 1, char *comment = "No comment", int drawtype = 0,
		 Float_t range_rms_l = 0.5, Float_t range_rms_h = 2.5,   // RMS plot's range
		 Float_t range_bsl_l = 5.0, Float_t range_bsl_h = 15.0,  // Baseline plot's range
		 char *prefix="bla" )
{

  gStyle->SetOptStat(0);
  gStyle->SetPadTopMargin(0.01);
  gStyle->SetPadLeftMargin(0.1);
  gStyle->SetPadRightMargin(0.15);
  gStyle->SetPadBottomMargin(0.1);
  gStyle->SetPalette(1,0);
  gStyle->SetPaintTextFormat("1.2f");

  layer_histograms *rms_layer[6];
  for ( int ilayer=0; ilayer<6; ilayer++ ) {
      rms_layer[ilayer] = new layer_histograms();
  }

  printf("==========================================================\n");
  SM_data *sm = &(e->sm[1]);

  for ( int ilayer=0; ilayer<6; ilayer++ ) {
    for ( int istack=0; istack<5; istack++ ) {

      int iroc = istack*6+ilayer;
      int chambertype = (istack == 2) ? 0 : 1;

      float mean_roc = sm->roc[iroc].mean;
      float rms_roc  = sm->roc[iroc].rms;

      // Fill histogram
      Double_t max_rob, min_rob, max_mcm, min_mcm;

      for( int irob = 0 ; irob < (2*(chambertype+3)); irob++ ) {

	Double_t rms_rob = sm->roc[iroc].rob[irob].rms;
	Double_t mean_rob = sm->roc[iroc].rob[irob].mean;
	max_rob = sm->roc[iroc].rob[irob].max;
	min_rob = sm->roc[iroc].rob[irob].min;

	Double_t nmcm = 0;
	for( int imcm = 0 ; imcm < 16; imcm++ ) {

	  Double_t rms_mcm  = sm->roc[iroc].rob[irob].mcm[imcm].rms;
	  Double_t mean_mcm = sm->roc[iroc].rob[irob].mcm[imcm].mean;
	  max_mcm = sm->roc[iroc].rob[irob].mcm[imcm].max;
	  min_mcm = sm->roc[iroc].rob[irob].mcm[imcm].min;

	  for( int iadc = 0 ; iadc < 21; iadc++ ) {
	    Double_t rms  = sm->roc[iroc].rob[irob].mcm[imcm].adc[iadc].rms;
	    Double_t mean = sm->roc[iroc].rob[irob].mcm[imcm].adc[iadc].mean;
	    Double_t p2p  = sm->roc[iroc].rob[irob].mcm[imcm].adc[iadc].peaktopeak;
	    rms_layer[ilayer]->FillMeanRmsP2P(istack, imcm, irob, iadc, mean, rms, p2p );
	  }

	  rms_layer[ilayer]->FillMcmRob(istack, imcm, irob, rms_mcm);

	}
      }

      printf("L%1dS%1d: <Baseline>= %.2f, <RMS>=%.2f\n", ilayer, istack, mean_roc, rms_roc );
      printf("ROB: Max=%.2f, Min=%.2f;   ", max_rob, min_rob);
      printf("MCM: Max=%.2f, Min=%.2f\n", max_mcm, min_mcm);
      printf("==========================================================\n");
    }
  }

  printf("\n");
  printf("==========================================================\n");
  printf("SM average: <Baseline>= %.2f, <RMS>=%.2f\n", sm->mean, sm->rms );
  printf("==========================================================\n");

  TCanvas *c = new TCanvas( "c", "o32_sm", 0, 0, 1400, 1000 );
  TPad *p_title = new TPad( "p_title", "p_title", 0, 0.97, 1.0, 1.0 );  p_title->Draw();
  TPad *p_main  = new TPad( "p_main",  "p_main", 0, 0, 1.0, 0.97 );     p_main->Draw();

  make_titlepad_sm( p_title );

  p_main->cd();
  p_main->Divide(6);

  for (int ilayer = 0; ilayer < 6; ilayer++) {

      p_main->cd(ilayer+1);
      MakePad(gPad);
      gPad->SetBorderMode(0);
      gPad->SetBorderSize(0);
      gPad->SetBottomMargin(0.);
      gPad->SetLeftMargin(0.0);
      gPad->SetRightMargin(0.);
      gPad->SetTopMargin(0.);


      for (int istack = 0; istack < 5; istack++) {
	  gPad->cd(5-istack);
	  if (drawtype == 0 ) {  
	      rms_layer[ilayer]->DrawRob(istack);
	      DrawAxesRob(gPad);
	  } else if ( drawtype == 1 ) {
	      rms_layer[ilayer]->DrawMcm(istack);
	      DrawAxesMcm(gPad);
	  } else if ( drawtype == 2 ) {
	      rms_layer[ilayer]->DrawRms(istack);
	      DrawAxesCh( gPad, "RMS (Noise/Channel)", 168);
	  } else if ( drawtype == 3 ) {
	      rms_layer[ilayer]->DrawMean(istack);
	      DrawAxesCh( gPad, "Mean (baseline)", 168);
	  } else if ( drawtype == 4 ) {
	      rms_layer[ilayer]->DrawPad(istack);
	      DrawAxesCh( gPad, "RMS (Noise/Pad)", 144);
	  } else if ( drawtype == 5 ) {
	      rms_layer[ilayer]->DrawP2P(istack);
	      DrawAxesMcm(gPad);
	  }
      }
  }

  // Save graphics file
  c->Print( "sm.png" );
}

// 
void make_titlepad( TPad *p_title, char *fname, int nevents, char *roc_name, char *comment )
{
  char tmp_string[1024];

  p_title->cd();

  TText *t_title = new TText( 0.02, 0.2, roc_name );
  t_title->SetTextSize( 0.8 );
  t_title->Draw();

  sprintf( tmp_string, "Comment: %s", comment );
  TText *t_comment = new TText( 0.3, 0.1, tmp_string );
  t_comment->SetTextSize( 0.4 );
  t_comment->Draw();

  sprintf( tmp_string, "File: %s", fname );
  TText *t_fname = new TText( 0.3, 0.55, tmp_string );
  t_fname->SetTextSize( 0.4 );
  t_fname->Draw();

}

// 
void make_titlepad_sm( TPad *p_title )
{
  char tmp_string[1024];

  p_title->cd();

  TText *t0 = new TText( 0.06, 0.2, "Layer 0" );
  t0->SetTextSize( 0.5 );
  t0->Draw();

  TText *t1 = new TText( 0.23, 0.2, "Layer 1" );
  t1->SetTextSize( 0.5 );
  t1->Draw();

  TText *t2 = new TText( 0.4, 0.2, "Layer 2" );
  t2->SetTextSize( 0.5 );
  t2->Draw();

  TText *t3 = new TText( 0.56, 0.2, "Layer 3" );
  t3->SetTextSize( 0.5 );
  t3->Draw();

  TText *t4 = new TText( 0.71, 0.2, "Layer 4" );
  t4->SetTextSize( 0.5 );
  t4->Draw();

  TText *t5 = new TText( 0.89, 0.2, "Layer 5" );
  t5->SetTextSize( 0.5 );
  t5->Draw();

}

//
int read_one_ev_ROC( unsigned int *evdata, FILE *f, int ilayer = 0, int istack = 0, int bside = 0) 
{
  int n = 0;
  char buf[256];
  unsigned int x;
  // Search next end-of-tracklet marker
  while( fgets( buf, 255, f ) != NULL ) {
    sscanf(buf, "0x%x", &x);
    if( x == 0xaaaaaaaa ) {
      //printf("End-of-Tracklet\n");
      break;
    }
  }
  unsigned int tmp, iRest, iDCS, iSM, iL, iS, iAB;
  while( fgets( buf, 255, f ) != NULL ) {
  sscanf(buf, "0x%x", &x);
    tmp = (x & 0x00000003);
    if( tmp == 1 ) {                  // 2 lsb of new header should be 01!
      iDCS  = (x & 0xfff00000) >> 20;
      iSM   = (x & 0x000f8000) >> 15;
      iL    = (x & 0x00007000) >> 12;
      iS    = (x & 0x00000e00) >> 9;
      iAB   = (x & 0x00000100) >> 8;
      iRest = (x & 0x000000ff);

      // temporary PATCH:                                          <== !!!!!!!!
      /*
      if (iDCS==2 && iSM==1  && iL==5 && iS==0 && iAB==0)  { iDCS=525; iSM=1; iL=0; iS=2; iAB=1; }
      if (iDCS==2 && iSM==2  && iL==4 && iS==0 && iAB==0)  { iDCS=532; iSM=1; iL=1; iS=2; iAB=1; }
      if (iDCS==1 && iSM==19 && iL==2 && iS==0 && iAB==0)  { iDCS=410; iSM=1; iL=2; iS=2; iAB=1; }
      if (iDCS==1 && iSM==20 && iL==3 && iS==0 && iAB==0)  { iDCS=419; iSM=1; iL=3; iS=2; iAB=1; }
      if (iDCS==2 && iSM==2  && iL==5 && iS==0 && iAB==0)  { iDCS=533; iSM=1; iL=4; iS=2; iAB=1; }
      if (iDCS==1 && iSM==19 && iL==6 && iS==0 && iAB==0)  { iDCS=414; iSM=1; iL=5; iS=2; iAB=1; }
      */
      // end PATCH                                                 <== !!!!!!!!

      //printf("ROC header\n");   // Found a ROC header
      if ( (iL == ilayer) && ( iS == istack) && ( iAB == bside ) ) {
	  printf("Found ROC: DCS%d, SM%02d, L%01dS%01d", iDCS, iSM, iL, iS);
	  if (iAB == 0) printf(" A side\n"); else  printf(" B side\n");
	  break;   // Found desired ROC
      }
    }
  }
    //evdata[n++] = x;
  // Copy till end of event
  while( fgets( buf, 255, f ) != NULL ) {
    sscanf(buf, "0x%x", &x);
    if( x == 0x00000000 ) break;  // end-of-event
    tmp = (x & 0x00000003);
    if( tmp == 1 ) break;    // 2 lsb of new header should be 01!
    evdata[n++] = x;
  }
  //printf("n=%d\n", n);
  return n;
}



void MakePad( TVirtualPad *p ) {

  p->cd();
  p->Range( -1, -6., 8.8, 83 );

  TFrame  *frame_1  = new TFrame(0, 0, 8, 80);
  frame_1->SetBorderSize(1);
  frame_1->Draw();

  return;

}

void DrawAxesCh( TVirtualPad *p , char *title, int limit = 168) {

  p->cd();

  // left axes
  TGaxis *axis_l1 = new TGaxis( 0,  0,  0, 15.9, -0.5, 15.5, 510, "-"); axis_l1->Draw();
  TGaxis *axis_l2 = new TGaxis( 0, 17,  0, 32.9, -0.5, 15.5, 510, "-"); axis_l2->Draw();
  TGaxis *axis_l3 = new TGaxis( 0, 34,  0, 45.9, -0.5, 11.5, 510, "-"); axis_l3->Draw();
  TGaxis *axis_l4 = new TGaxis( 0, 47,  0, 62.9, -0.5, 15.5, 510, "-"); axis_l4->Draw();
  TGaxis *axis_l5 = new TGaxis( 0, 64,  0, 79.9, -0.5, 15.5, 510, "-"); axis_l5->Draw();

  // bottom axis
  TGaxis *axis_b = new TGaxis( 0,   0,  8., 0., -0.5, limit-0.5, 510, ""); axis_b->Draw();
  axis_b->SetLabelOffset(0.);
  axis_b->SetTickSize(0.01);

  TLine  *line_1 = new TLine( 0, 16, 8., 16 ); line_1->Draw();
  TLine  *line_2 = new TLine( 0, 17, 8., 17 ); line_2->Draw();
  TLine  *line_3 = new TLine( 0, 33, 8., 33 ); line_3->Draw();
  TLine  *line_4 = new TLine( 0, 34, 8., 34 ); line_4->Draw();
  TLine  *line_5 = new TLine( 0, 46, 8., 46 ); line_5->Draw();
  TLine  *line_6 = new TLine( 0, 47, 8., 47 ); line_6->Draw();
  TLine  *line_7 = new TLine( 0, 63, 8., 63 ); line_7->Draw();
  TLine  *line_8 = new TLine( 0, 64, 8., 64 ); line_8->Draw();
  TLine  *line_9 = new TLine( 0, 80, 8., 80 ); line_9->Draw();

  TLine *xline[7][5];
  for( int i = 0 ; i < 7 ; i++ ) {
    for( int j = 0 ; j < 2 ; j++ ) {
      xline[i][j] = new TLine( i+1., j*17, i+1., (j+1)*16 + j*1);
      xline[i][j]->Draw();
      xline[i][j+3] = new TLine( i+1., 47 + j*17, i+1., 47 + (j+1)*16 + j*1);
      xline[i][j+3]->Draw();
    }
    xline[i][j] = new TLine( i+1., 34, i+1., 46);
    xline[i][j]->Draw();
  }

  TLine *yline[15][5];
  for( int j = 0 ; j < 2 ; j++ ) {
    for( int i = 0 ; i < 15 ; i++ ) { 
      yline[i][j] = new TLine( 0., j*17 + i + 1, 8., j*17 + i + 1);
      yline[i][j]->Draw();
      yline[i][j+3] = new TLine( 0., 47 + j*17 + i + 1, 8., 47 + j*17 + i + 1);
      yline[i][j+3]->Draw();
    }
  }
  for( int i = 0 ; i < 11 ; i++ ) { 
    yline[i][2] = new TLine( 0., 34 + i + 1, 8., 34 + i + 1 );
    yline[i][2]->Draw();
  }

  TText *titletext = new TText( 0, 80.5, title); titletext->Draw();

  TText *xtitle = new TText( 4., -2, "Pad Number in Phi" );
  xtitle->SetTextSize( 0.05 );
  xtitle->Draw();

  TText *ytitle = new TText();
  ytitle->SetTextSize( 0.05 );
  ytitle->SetTextAngle(90);
  ytitle->DrawText( -0.5, 69,  "Stack 4" );
  ytitle->DrawText( -0.5, 52,  "Stack 3" );
  ytitle->DrawText( -0.5, 38,  "Stack 2" );
  ytitle->DrawText( -0.5, 22,  "Stack 1" );
  ytitle->DrawText( -0.5, 5,  "Stack 0" );

  //TText *marker = new TText(   6, 80.5, "Noise per Channel" );
  //marker
  //marker->Draw();

}


void DrawAxesMcm( TVirtualPad *p ) {

  p->cd();

  // left axes
  TGaxis *axis_l1 = new TGaxis( 0,  0,  0, 15.9, -0.5, 15.5, 510, "-"); axis_l1->Draw();
  TGaxis *axis_l2 = new TGaxis( 0, 17,  0, 32.9, -0.5, 15.5, 510, "-"); axis_l2->Draw();
  TGaxis *axis_l3 = new TGaxis( 0, 34,  0, 45.9, -0.5, 11.5, 510, "-"); axis_l3->Draw();
  TGaxis *axis_l4 = new TGaxis( 0, 47,  0, 62.9, -0.5, 15.5, 510, "-"); axis_l4->Draw();
  TGaxis *axis_l5 = new TGaxis( 0, 64,  0, 79.9, -0.5, 15.5, 510, "-"); axis_l5->Draw();

  // bottom axis
  TGaxis *axis_b = new TGaxis( 0,   0,  8.,  0., -0.5,  7.5,1010, ""); axis_b->Draw();
  axis_b->SetLabelOffset(0.);

  TLine  *line_1 = new TLine( 0, 16, 8., 16 ); line_1->Draw();
  TLine  *line_2 = new TLine( 0, 17, 8., 17 ); line_2->Draw();
  TLine  *line_3 = new TLine( 0, 33, 8., 33 ); line_3->Draw();
  TLine  *line_4 = new TLine( 0, 34, 8., 34 ); line_4->Draw();
  TLine  *line_5 = new TLine( 0, 46, 8., 46 ); line_5->Draw();
  TLine  *line_6 = new TLine( 0, 47, 8., 47 ); line_6->Draw();
  TLine  *line_7 = new TLine( 0, 63, 8., 63 ); line_7->Draw();
  TLine  *line_8 = new TLine( 0, 64, 8., 64 ); line_8->Draw();
  TLine  *line_9 = new TLine( 0, 80, 8., 80 ); line_9->Draw();

  TLine *xline[7][5];
  for( int i = 0 ; i < 7 ; i++ ) {
    for( int j = 0 ; j < 2 ; j++ ) {
      xline[i][j] = new TLine( i+1., j*17, i+1., (j+1)*16 + j*1);
      xline[i][j]->Draw();
      xline[i][j+3] = new TLine( i+1., 47 + j*17, i+1., 47 + (j+1)*16 + j*1);
      xline[i][j+3]->Draw();
    }
    xline[i][j] = new TLine( i+1., 34, i+1., 46);
    xline[i][j]->Draw();
  }

  TLine *yline[15][5];
  for( int j = 0 ; j < 2 ; j++ ) {
    for( int i = 0 ; i < 15 ; i++ ) { 
      yline[i][j] = new TLine( 0., j*17 + i + 1, 8., j*17 + i + 1);
      yline[i][j]->Draw();
      yline[i][j+3] = new TLine( 0., 47 + j*17 + i + 1, 8., 47 + j*17 + i + 1);
      yline[i][j+3]->Draw();
    }
  }
  for( int i = 0 ; i < 11 ; i++ ) { 
    yline[i][2] = new TLine( 0., 34 + i + 1, 8., 34 + i + 1 );
    yline[i][2]->Draw();
  }

  TText *titletext = new TText( 0, 80.5, "RMS Distribution (MCM)"); titletext->Draw();

  TText *xtitle = new TText( 4., -2, "MCM Number in Phi" );
  xtitle->SetTextSize( 0.05 );
  xtitle->Draw();

  TText *ytitle = new TText();
  ytitle->SetTextSize( 0.05 );
  ytitle->SetTextAngle(90);
  ytitle->DrawText( -0.5, 69,  "Stack 4" );
  ytitle->DrawText( -0.5, 52,  "Stack 3" );
  ytitle->DrawText( -0.5, 38,  "Stack 2" );
  ytitle->DrawText( -0.5, 22,  "Stack 1" );
  ytitle->DrawText( -0.5, 5,  "Stack 0" );

  //TText *marker = new TText(   6, 80.5, "Noise per MCM" );
  //marker->SetTextSize( 0.05);
  //marker->Draw();

}


void DrawAxesRob( TVirtualPad *p ) {

  p->cd();

  // left axes
  TGaxis *axis_l1 = new TGaxis( 0,  0,  0, 15.9, -0.5, 3.5, 405, "-"); axis_l1->Draw();
  TGaxis *axis_l2 = new TGaxis( 0, 17,  0, 32.9, -0.5, 3.5, 405, "-"); axis_l2->Draw();
  TGaxis *axis_l3 = new TGaxis( 0, 34,  0, 45.9, -0.5, 2.5, 405, "-"); axis_l3->Draw();
  TGaxis *axis_l4 = new TGaxis( 0, 47,  0, 62.9, -0.5, 3.5, 405, "-"); axis_l4->Draw();
  TGaxis *axis_l5 = new TGaxis( 0, 64,  0, 79.9, -0.5, 3.5, 405, "-"); axis_l5->Draw();

  // bottom axis
  TGaxis *axis_b = new TGaxis( 0,   0,  8.,  0., -0.5, 1.5, 200, ""); axis_b->Draw();
  axis_b->SetLabelOffset(0.);

  TLine  *line_1 = new TLine( 0, 16, 8., 16 ); line_1->Draw();
  TLine  *line_2 = new TLine( 0, 17, 8., 17 ); line_2->Draw();
  TLine  *line_3 = new TLine( 0, 33, 8., 33 ); line_3->Draw();
  TLine  *line_4 = new TLine( 0, 34, 8., 34 ); line_4->Draw();
  TLine  *line_5 = new TLine( 0, 46, 8., 46 ); line_5->Draw();
  TLine  *line_6 = new TLine( 0, 47, 8., 47 ); line_6->Draw();
  TLine  *line_7 = new TLine( 0, 63, 8., 63 ); line_7->Draw();
  TLine  *line_8 = new TLine( 0, 64, 8., 64 ); line_8->Draw();
  TLine  *line_9 = new TLine( 0, 80, 8., 80 ); line_9->Draw();

  TLine *xline[5];
  for( int j = 0 ; j < 2 ; j++ ) {
    xline[j] = new TLine( 4., j*17, 4., (j+1)*16 + j*1);
    xline[j]->Draw();
    xline[j+3] = new TLine( 4., 47 + j*17, 4., 47 + (j+1)*16 + j*1);
    xline[j+3]->Draw();
  }
  xline[j] = new TLine( 4., 34, 4., 46);
  xline[j]->Draw();

  TLine *yline[3][5];

  TText *text = new TText();
  text->SetTextSize(0.05);

  for( int j = 0 ; j < 2 ; j++ ) {
    for( int i = 0 ; i < 2 ; i++ ) { 
      text->DrawTextNDC(0.13, 0.225+j*0.185+i*0.517, "Pos 7");
      text->DrawTextNDC(0.55, 0.225+j*0.185+i*0.517, "Pos 6");
      text->DrawTextNDC(0.13, 0.185+j*0.185+i*0.517, "Pos 5");
      text->DrawTextNDC(0.55, 0.185+j*0.185+i*0.517, "Pos 4");
      text->DrawTextNDC(0.13, 0.140+j*0.185+i*0.517, "Pos 3");
      text->DrawTextNDC(0.55, 0.140+j*0.185+i*0.517, "Pos 2");
      text->DrawTextNDC(0.13, 0.095+j*0.185+i*0.517, "Pos 1");
      text->DrawTextNDC(0.55, 0.095+j*0.185+i*0.517, "Pos 0");
    }
    for( int i = 0 ; i < 3 ; i++ ) { 
      yline[i][j] = new TLine( 0., j*17 + (i + 1)*4, 8., j*17 + (i + 1)*4);
      yline[i][j]->Draw();
      yline[i][j+3] = new TLine( 0., 47 + j*17 + (i + 1)*4, 8., 47 + j*17 + (i + 1)*4);
      yline[i][j+3]->Draw();
    }
  }
  text->DrawTextNDC(0.13, 0.555, "Pos 5");
  text->DrawTextNDC(0.55, 0.555, "Pos 4");
  text->DrawTextNDC(0.13, 0.510, "Pos 3");
  text->DrawTextNDC(0.55, 0.510, "Pos 2");
  text->DrawTextNDC(0.13, 0.465, "Pos 1");
  text->DrawTextNDC(0.55, 0.465, "Pos 0");
  for( int i = 0 ; i < 2 ; i++ ) { 
    yline[i][2] = new TLine( 0., 34 + (i + 1)*4, 8., 34 + (i + 1)*4);
    yline[i][2]->Draw();
  }

  TText *titletext = new TText( 0, 80.5, "RMS Distribution (ROB)"); titletext->Draw();

  TText *xtitle = new TText( 0.5, -2, "Rob Position in Phi (B and A side)" );
  xtitle->SetTextSize( 0.05 );
  xtitle->Draw();

  TText *ytitle = new TText();
  ytitle->SetTextSize( 0.05 );
  ytitle->SetTextAngle(90);
  ytitle->DrawText( -0.5, 69,  "Stack 4" );
  ytitle->DrawText( -0.5, 52,  "Stack 3" );
  ytitle->DrawText( -0.5, 38,  "Stack 2" );
  ytitle->DrawText( -0.5, 22,  "Stack 1" );
  ytitle->DrawText( -0.5, 5,  "Stack 0" );

  //TText *marker = new TText(   6, 80.5, "Noise per ROB" );
  //marker->SetTextSize( 0.05);
  //marker->Draw();

}

class layer_histograms {

public:

  float shift_y[5];

  char str1[256];
  char str2[256];
  
  TH2F *rms[5];          // RMS  map with highest granurality
  TH2F *mean[5];         // Mean map with highest granurality
  TH2F *rms_mcm[5];      // RMS  map with MCM-by-MCM granurality
  TH2F *rms_pad[5];      // RMS  map pad-to-pad
  TH2F *rms_rob[5];      // RMS  map with ROB-by-ROB granurality
  TH2F *p2p[5];          // Peak to peak map with highest granurality
  
  // class member function entities are at end of this file
  layer_histograms();
  ~layer_histograms();
  void FillMeanRmsP2P( int istack, int imcm, int irob, int iadc, double Mean, double RMS, double p2p );
  void FillMcmRob(  int istack, int imcm, int irob, double RMS_rob );
  void DrawMean( int istack );
  void DrawRms(  int istack );
  void DrawPad(  int istack );
  void DrawMcm(  int istack );
  void DrawRob(  int istack );
  void DrawP2P(  int istack );

};

class c_histograms {
public:
  TH2F *rms;          // RMS  map with highest granurality
  TH2F *mean;         // Mean map with highest granurality
  TH2F *rms_mcm;      // RMS  map with MCM-by-MCM granurality
  TH2F *rms_rob;      // RMS  map with ROB-by-ROB granurality
  TH1F *rms_pads;     // RMS map with trace specific features
  TH2F *p2p;          // Peak to peak map with highest granurality

  c_histograms( Float_t range_rms_l, Float_t range_rms_h, Float_t range_bsl_l, Float_t range_bsl_h ) {

    rms       = new TH2F( "h_rms",       "RMS distribution in channels",     168, -0.5, 167.5, 16, -0.5, 15.5 );
    mean      = new TH2F( "h_mean",      "Mean distribution of baseline",    168, -0.5, 167.5, 16, -0.5, 15.5 );
    rms_mcm   = new TH2F( "h_rms_mcm",   "RMS distribution (MCM)",             8, -0.5,   7.5, 16, -0.5, 15.5 );
    rms_rob   = new TH2F( "h_rms_rob",   "RMS distribution (ROB)",             2, -0.5,   1.5,  4, -0.5,  3.5 );
    rms_pads  = new TH1F( "h_rms_pads",  "RMS distribution (Pads)",          144, -0.5, 143.5 );
    p2p       = new TH2F( "h_p2p",  "Peak to Peak distribution in channels", 168, -0.5, 167.5, 16, -0.5, 15.5 );

    rms->SetXTitle("ADC channel number in phi");
    rms->SetYTitle("Chamber Pad number in z");
    rms->SetZTitle("RMS");
    rms->SetAxisRange(  -10, 168, "X");
    rms->SetAxisRange(  -10,  16, "Y");
    rms->SetAxisRange( range_rms_l, range_rms_h, "Z");

    mean->SetXTitle("ADC channel Number in phi");
    mean->SetYTitle("Chamber Pad Number in z");
    mean->SetZTitle("Mean");
    mean->SetAxisRange( -10, 168, "X");
    mean->SetAxisRange( -10,  16, "Y");
    mean->SetAxisRange( range_bsl_l, range_bsl_h, "Z");

    rms_mcm->SetXTitle("MCM Number in phi");
    rms_mcm->SetYTitle("MCM Number in z");
    rms_mcm->SetZTitle("<RMS>");
    rms_mcm->SetAxisRange( range_rms_l, range_rms_h, "Z");
    rms_mcm->SetMarkerSize(2);

    rms_rob->SetXTitle("ROB Number in phi");
    rms_rob->SetYTitle("ROB Number in z");
    rms_rob->SetZTitle("<RMS>");
    rms_rob->SetAxisRange( range_rms_l, range_rms_h, "Z");
    rms_rob->SetMarkerSize(3);

    rms_pads->SetXTitle("ADC channel number in phi");
    rms_pads->SetYTitle("RMS");
    rms_pads->SetAxisRange( -10, 168, "X");
    rms_pads->SetAxisRange(0.05, 12., "Y");
    rms_pads->SetFillColor(2);

    p2p->SetXTitle("ADC channel number in phi");
    p2p->SetYTitle("Chamber Pad number in z");
    p2p->SetZTitle("RMS");
    p2p->SetAxisRange(  -10, 168, "X");
    p2p->SetAxisRange(  -10,  16, "Y");
    p2p->SetAxisRange( range_rms_l, range_rms_h, "Z");

  }
};

// Class constructor
layer_histograms::layer_histograms() {

    //printf("layer_histograms:: Generating..\n");

    float shift_tmp[5] = {0., 17., 34., 47., 64.};

    for (int istack = 0; istack < 5; istack++) {
    
	shift_y[istack] = shift_tmp[istack];

	if (istack == 2) {
	    sprintf( str1, "h_rms%1d", istack);
	    rms[istack]      = new TH2F( str1, "", 168, 0., 8., 12, shift_y[istack],
					 12.+shift_y[istack] );
	    sprintf( str1, "h_rms_pad%1d", istack);
	    rms_pad[istack]  = new TH2F( str1, "", 144, 0., 8., 12, shift_y[istack],
					 12.+shift_y[istack] );
	    sprintf( str1, "h_mean%1d", istack);
	    mean[istack]     = new TH2F( str1, "", 168, 0., 8., 12, shift_y[istack],
					 12.+shift_y[istack] );
	    sprintf( str1, "h_rms_mcm%1d", istack);
	    rms_mcm[istack]  = new TH2F( str1, "", 8, 0., 8., 12, shift_y[istack],
					 12.+shift_y[istack] );
	    sprintf( str1, "h_rms_rob%1d", istack);
	    rms_rob[istack]  = new TH2F( str1, "", 2, 0., 8.,  3, shift_y[istack],
					 12.+shift_y[istack] );
	    sprintf( str1, "h_p2p%1d", istack);
	    p2p[istack]      = new TH2F( str1, "", 168, 0., 8., 12, shift_y[istack],
					 12.+shift_y[istack] );
	} else {
	    sprintf( str1, "h_rms%1d", istack);
	    rms[istack]      = new TH2F( str1, "", 168, 0., 8., 16, shift_y[istack],
					 16.+shift_y[istack] );
	    sprintf( str1, "h_rms_pad%1d", istack);
	    rms_pad[istack]  = new TH2F( str1, "", 144, 0., 8., 16, shift_y[istack],
					 16.+shift_y[istack] );
	    sprintf( str1, "h_mean%1d", istack);
	    mean[istack]     = new TH2F( str1, "", 168, 0., 8., 16, shift_y[istack],
					 16.+shift_y[istack] );
	    sprintf( str1, "h_rms_mcm%1d", istack);
	    rms_mcm[istack]  = new TH2F( str1, "", 8, 0., 8., 16, shift_y[istack],
					 16.+shift_y[istack] );
	    sprintf( str1, "h_rms_rob%1d", istack);
	    rms_rob[istack]  = new TH2F( str1, "", 2, 0., 8.,  4, shift_y[istack],
					 16.+shift_y[istack] );
	    sprintf( str1, "h_p2p%1d", istack);
	    p2p[istack]      = new TH2F( str1, "", 168, 0., 8., 16, shift_y[istack],
					 16.+shift_y[istack] );
	}
    
	rms[istack]->SetZTitle("RMS");
	rms[istack]->SetTitleOffset(0.4,"Z");
	rms[istack]->SetTitleOffset(0.4,"X");
	rms[istack]->SetAxisRange( 0.4, 2.8, "Z");
    
	rms_pad[istack]->SetZTitle("RMS");
	rms_pad[istack]->SetTitleOffset(0.4,"Z");
	rms_pad[istack]->SetAxisRange( 0.4, 2.8, "Z");
    
	mean[istack]->SetZTitle("Mean");
	mean[istack]->SetAxisRange( 9., 11., "Z");
    
	rms_mcm[istack]->SetZTitle("<RMS>");
	rms_mcm[istack]->SetAxisRange( 0.4, 2.8, "Z");
	rms_mcm[istack]->SetMarkerSize(1.5);
    
	rms_rob[istack]->SetZTitle("<RMS>");
	rms_rob[istack]->SetAxisRange( 0.4, 2.8, "Z");
	rms_rob[istack]->SetMarkerSize(3);
    
	p2p[istack]->SetZTitle("RMS");
	p2p[istack]->SetTitleOffset(0.4,"Z");
	p2p[istack]->SetTitleOffset(0.4,"X");
	p2p[istack]->SetAxisRange( 0., 20., "Z");

    }

    return;

}


// Class member functions

void layer_histograms::FillMeanRmsP2P( int istack, int imcm, int irob, int iadc, double Mean, double RMS, double P2P ) {
  Double_t x = (1-(irob%2)) *4*21 + (3-(imcm%4))*21 + iadc;
  Double_t y = (irob/2)*4 + imcm/4;
  Double_t pad = (1-(irob%2)) *4*18 + (3-(imcm%4))*18 + iadc - 2;
  if ( iadc < 2 )  pad = -1;
  if ( iadc > 19 ) pad = -1;
  rms[istack]    ->Fill( (x + 0.5)/21., y + shift_y[istack] + 0.5, RMS );
  rms_pad[istack]->Fill( (pad+0.5)/18., y + shift_y[istack] + 0.5, RMS );
  mean[istack]   ->Fill( (x + 0.5)/21., y + shift_y[istack] + 0.5, Mean );
  p2p[istack]    ->Fill( (x + 0.5)/21., y + shift_y[istack] + 0.5, P2P );
  return;
}

void layer_histograms::FillMcmRob( int istack , int imcm, int irob, double RMS_mcm ) {
  rms_mcm[istack]->Fill( (1-(irob%2))*4 + (3-(imcm%4)) + 0.5,
			 (irob/2)*4 + imcm/4 + shift_y[istack] + 0.5,
			 RMS_mcm );
  rms_rob[istack]->Fill( (1-(irob%2))*4 + (3-(imcm%4)) + 0.5,
			 (irob/2)*4 + imcm/4 + shift_y[istack] + 0.5,
			 RMS_mcm / 16. );
  return;
}

void layer_histograms::DrawMean( int istack ) {
  mean[istack]->Draw("samecolz");
  return;
}

void layer_histograms::DrawRms( int istack ) {
  rms[istack]->Draw("samecolz");
  return;
}

void layer_histograms::DrawMcm( int istack ) {
  rms_mcm[istack]->Draw("samecolz");
  rms_mcm[istack]->Draw("sametext");
  return;
}

void layer_histograms::DrawPad( int istack ) {
  rms_pad[istack]->Draw("samecolz");
  return;
}

void layer_histograms::DrawRob( int istack ) {
  rms_rob[istack]->Draw("samecolz");
  rms_rob[istack]->Draw("sametext");
  return;
}

void layer_histograms::DrawP2P( int istack ) {
  p2p[istack]->Draw("samecolz");
  return;
}


// Find automatically high noise channels and high pedestal channels.
//
void find_bad_channels( c_histograms *h, Double_t rms_thresh, Double_t mean_thresh ) 
{
  printf("Scanning channels.\n");
  printf("Threshold: %.2f for RMS\n",     rms_thresh );
  printf("Threshold: %.2f for Pedestal\n", mean_thresh );

  printf("// begin tcs\n" );

  for( int irob = 0 ; irob < 8; irob++ ) {
    for( int imcm = 0 ; imcm < 16; imcm++ ) {
      int ADCMSK = 0x1FFFFF;
      if( ((irob % 2) == 0 ) && ((imcm % 4) == 0)) ADCMSK = 0x0FFFFF;
      if( ((irob % 2) == 1 ) && ((imcm % 4) == 3)) ADCMSK = 0x1FFFFC;
      int modify = 0;
      for( int iadc = 0 ; iadc < 21; iadc++ ) {
	Int_t x = (1-(irob%2)) *4*21 + (3-(imcm%4))*21 + iadc +1;
	Int_t y = (irob/2)*4 + imcm/4 +1;
	Double_t rms  = h->rms ->GetBinContent( x, y );
	Double_t mean = h->mean->GetBinContent( x, y );
	if( rms > rms_thresh ) {
	  modify ++;
	  ADCMSK = ADCMSK & (~(1 << iadc));
	  printf("// highnoise: irob=%d, imcm=%2d iadc=%d rms=%.2f\n", irob, imcm, iadc, rms);
	}
	if( mean > mean_thresh ) {
	  modify ++;
	  ADCMSK = ADCMSK & (~(1 << iadc));
	  printf("// highnoise: irob=%d, imcm=%2d iadc=%d mean=%.2f\n", irob, imcm, iadc, mean);
	}

      }
      if( modify != 0 ) {
	printf("write chip_rob%d | chip%d, ADCMSK, 0x%x;\n", irob, imcm, ADCMSK);
      }
    }
  }
  return;
}

// EOF
