#!/bin/sh

###
# Maximum time to wait for neighbour
MAXTIME=45

if [ -z $1 ]; then
    echo "usage: $0 <Master DCS Board> [recursive]"
    echo "[recursive] (optional) If nextboard is ok and accessible, continue testing"
    exit 1
fi

if [ ${2}x = "recursivex" ]; then
  RECUR="TRUE"
# Third paramter is set only in recursive calls to indicate first board
  if [ -z $3 ]; then
    FIRSTBOARD=$1
  else
    FIRSTBOARD=$3
  fi
  echo "-----------------------------------------------------------------------------"
  echo "Recursive Testing with Master: $1, Firstboard: $FIRSTBOARD"
else
  RECUR="FALSE"
fi
ME_MYSELF_AND_I=$0
REMIP=$1
REMOTE=root@$1
TMPFILE1="/tmp/testjtag_tmpfile1_$$"
TMPFILE2="/tmp/testjtag_tmpfile2_$$"

############################################################################
# check if board alive

ping -c 1 -w 3 $REMIP 2>&1 > /dev/null
if [ $? != 0 ]; then
    echo "No ping response from Masterboard ${REMIP}."
    exit 1
else
    echo "Master Board $REMIP is alive."
fi

############################################################################

ssh $REMOTE 'echo -e "cable file /dev/jtag\ndetect\nquit" > /tmp/jtagscr ; /usr/local/sbin/jtag /tmp/jtagscr' > $TMPFILE1

grep "Part:" $TMPFILE1
grep "Chain length: 3" $TMPFILE1
if [ $? -ne 0 ]; then 
    echo "Error: No or wrong JTAG chain detected (3 Parts expected)"
    exit 1
fi
echo "Chain ok - retrieving Boardnumber..."

ssh $REMOTE 'echo -e "cable file /dev/jtag\ndetect\ninitbus epxa1\ndetectflash\nreadmem 0x40000 256 /tmp/nextboardenv" > /tmp/jtagscr ; /usr/local/sbin/jtag /tmp/jtagscr' > $TMPFILE1
scp $REMOTE:/tmp/nextboardenv $TMPFILE2
if [ $? -ne 0 ]; then 
    echo "Error: Could not get next boards environment data"
    exit 1
fi

filesboardnum=`strings $TMPFILE2  | sed "s/ /\n/g" | grep "easynet_mac" | sed 's/^.*=//' | head -1 | sed 's/^..:..:..://' | sed 's/://g' | sed "s/.*/0x&/" | xargs printf "%d"`
NEXTBOARDIP=`printf "dcs%04d" "$filesboardnum"`
echo "Nextboard : $NEXTBOARDIP"

###############################

#echo "Power cycle $NEXTBOARDIP"
#ssh $REMOTE "/usr/local/sbin/nextboard 2 > /dev/null ; sleep 2; /usr/local/sbin/nextboard 0 > /dev/null"
#echo "Waiting for Boot of $NEXTBOARDIP..."

MAXTIME=45
RET=1
STARTSECS=`date +%s`
COUNT=0
#while([ $COUNT -lt $MAXTIME -a $RET -ne 0 ])
#do
#    ping -c 1 -w 3 $NEXTBOARDIP 2>&1 > /dev/null
#    RET=$?
#    nowsecs=`date +%s`
#    # echo "Second $((nowsecs-STARTSECS))"
#    echo -n "."
#    COUNT=$((COUNT+1))
#done
#if [ $RET -ne 0 ]; then 
#    echo "$NEXTBOARDIP did not boot after $((nowsecs-STARTSECS)) seconds"
#    exit 1
#else
#    echo "$NEXTBOARDIP booted in $((nowsecs-STARTSECS)) seconds"
#fi

if [ $RECUR = "TRUE" ];    then
    if [ $NEXTBOARDIP = $FIRSTBOARD ]; then
	echo "Last connection tested, the whole ring is ok"
#	exit 0
    else
	echo "Starting myself again in 8 seconds..."
	sleep 8
	$ME_MYSELF_AND_I $NEXTBOARDIP recursive $FIRSTBOARD
	exit $?
    fi
fi

rm -f $TMPFILE1 
rm -f $TMPFILE2

exit 0
